#!/bin/sh
# $Id$
#
# Script:	$RCSfile$
#
# Description:	Generic RSF-1 agent header file
#
# Platform:	Unix
#
# Authors:	High-Availability.Com
#
PRODUCT="RSF-1"		        # main product name
PRODUCT_NAME="RSF-1"		# product script tag
PRODUCT_PREFIX="agents"		# product filename prefix for binaries
PRODUCT_SCRIPT="agents.sh"	# product script definition file - this filename
PRODUCT_BINARY="agents"		# main product binary filename, that checks licensing

. /opt/HAC/bin/rsf.sh

RSF_AGENTDIR="${PRODUCT_DIR}/agents"
RSF_ACTIONDIR="${PRODUCT_AGENTDIR}/actions"

#agent support functions

#agent_set_param(varname,value)
#  sets a variable to a value
agent_set_param()
{
    if [ $# -ne 2 ]; then
	dated_echo "Usage:agent_set_param varname value" >&1
	exit
    fi

    VAR=${1}

    eval ${VAR}=${2}
}

#agent_check_param(varname)
#  returns if a variable is null or not
agent_check_param()
{
    if [ $# -ne 1 ]; then
	dated_echo "Usage:agent_check_param varname" >&1
	return 1
    fi

    VAR=${1}

    #VAR=${!VAR}   # de-reference the variable - method only worked in bash
    VAR=`eval echo \$\{${VAR}\}` # de-reference the variable

    if [ -z "${VAR}" ]; then
	return 1
    fi

    return 0
}

# agent_param(varname, error text)
#  checks for varname being set ie not null/unset else prints error text and
#  exits
agent_param()
{
    if [ $# -ne 2 ]; then
	dated_echo "Usage:agent_param varname errortext" >&1
	exit
    fi

    agent_check_param $1

    if [ $? -ne 0 ]; then
	dated_echo ${2} >&1
	exit
    fi
}

#agent_default_param(varname,defaultvalue)
#  checks if a variable is empty and if so assigns a default
agent_default_param()
{
    if [ $# -ne 2 ]; then
	dated_echo "Usage:agent_default_param varname defaultvalue" >&1
	exit
    fi

    agent_check_param $1

    if [ $? -ne 0 ]; then
	agent_set_param $1 $2
    fi
}

#
# Returns number of the named processes that are running.
#
agent_require_process()
{
    count=`ps -A -o comm | awk 'BEGIN { FS="/" } { print $NF }' | grep -c "$1"`
    return $count
}
