#!/bin/sh
#
# (c)2003 High-Availability.Com
#
# Script:	monitor.sh
#
# Description:	Shell script fragment for shell monitoring agents that run
#                 periodically
#
# Platforms:	All
#
# Author:	High-Availability.Com
#
# $Id$

#start monitor
# $1 is the agents name
# $2 is the agents poll time (in seconds)
# $3 is the agents monitor program
# $4 is the command to execute on failure
monitor_start()
{
	# check for existing PID file
	if [ -f "${pidfile}" ]; then
		dated_echo "Already running or stale ${pidfile} remains"
		exit 1
	fi

	dated_echo "Starting $1"
	# run continuously in background subshell
	(
	trap "rm -f ${pidfile}; exit" 2 15	# clean up on termination

	while true; do	# run forever
		sleep $2	# wait before next check

		if [ ! -f "/tmp/rsf_suspend_agents" ]; then
		        eval $3	# call monitor program
		        if [ $? -ne 0 ]; then	# exit loop on non zero
			        break
			fi
		fi
	done

	dated_echo "monitoring failed, executing $4"
	# action on failure
	eval $4	# action to take on failure
	rm -f ${pidfile}	# clean up
	) &
	echo $! > ${pidfile}	# save PID for killing later
}

#start monitor
# TODO - implement as a c program
# $1 is the agents name
# $2 is the agents poll time (in seconds)
monitor_stop()
{
	# kill running instance of monitor
	if [ -f "${pidfile}" ]; then
		dated_echo "Stopping $1"
		kill `cat ${pidfile}` >/dev/null 2>&1
	else
		dated_echo "Nothing to stop"
		exit ${RSF_WARN}
	fi
	sleep $2
}



#AGENT_NAME=                        # agents name - used in log messages
#AGENT_POLL=                        # poll interval (secs)
#AGENT_MONITOR=  	            # command to run to check application
#			     	    # non 0 exit code indicates failure
#AGENT_ACTION=                      # action on failure

#check necessary vars are defined
if [ -z "${AGENT_NAME}" -o -z "${AGENT_POLL}" \
     -o -z "${AGENT_MONITOR}" -o -z "${AGENT_ACTION}" ];then
     echo "Missing variables monitor script cannot continue!"
     exit ${RSF_WARN}
fi

if [ -n "${MONITOR_CONFIG}" ];then
    export MONITOR_CONFIG
fi

# internal vars
state="$1"
service=${RSF_SERVICE:-"unknown"}
pidfile="/tmp/${AGENT_NAME}-${service}.pid"

case "${state}" in
'start')
	monitor_start "${AGENT_NAME}" "${AGENT_POLL}" "${AGENT_MONITOR}" "${AGENT_ACTION}"
	;;

'stop')
	monitor_stop "${AGENT_NAME}" "${AGENT_POLL}"
	;;

*)
	dated_echo "Usage: $0 <start|stop>"
	exit ${RSF_WARN}
	;;

esac

exit ${RSF_OK}
