#!/bin/sh
SVCTEST="/opt/HAC/RSF-1/bin/is_service_running_on_host.py"
LOGFILE="/opt/HAC/RSF-1/log/vip_config.log"

print_help() {
   echo "
Usage: ${script} [options]
Options:
  -plumb              Plumb in the interface.
  -unplumb            Unplumb the interace.
  -device=<dev>       Specify the device to apply plumb/unplumb to.
  -vip=<vip>          The virtual hostname.
  -netmask=<mask>     The netmask to use when plumbing in an interface.
  -service=<service>  The service to apply. 
  -h                  Help - print this help text and exit.

Exit codes:
  0                   Successfully plumbed/unplumbed vip.
  1                   Service not running on node.     
  2                   Error occured.
"

exit 0
}

make_log() {
    echo "`date`: $*" >> ${LOGFILE}
}

remove_interface()
{
    count=0
    while [ 1 == 1  ] ; do
	make_log "attempt ${count}: ifconfig ${device} removeif ${vip}"
	ifconfig ${device} removeif ${vip} >>${LOGFILE} 2>&1 
	count=`expr ${count} + 1`
	if [ $? -ne 0 -o ${count} -ge 10 ] ; then
            break
	fi
    done
}

make_log "called with args $*"

do_plumb="no"
do_unplumb="no"
device=""
netmask=""
vip=""

while true; do
    case "$1" in
        -plumb)
	    do_plumb="yes"
            shift
            ;;

        -unplumb)
            do_unplumb="yes"
            shift
            ;;

	-device=*)
            device=`echo $1|sed 's/-device=//'`
	    shift
	    ;;

	-vip=*)
	    vip=`echo $1|sed 's/-vip=//'`
	    shift
	    ;;

	-netmask=*)
	    mask="`echo $1|sed 's/-netmask=//'`"
            if [ "${mask}" != "default" ] ; then 
	    	netmask="netmask ${mask}"
	    fi
	    shift
	    ;;

	-service=*)
            service=`echo $1|sed 's/-service=//'`
	    shift
	    ;;

        -h | -help)
            print_help
            exit 1
            ;;
        -*)
            echo "Unknown option $1, use ${script} -h for help."
            exit 1
            shift
            ;;
        *)
            break
            ;;
    esac
done

if [ "${do_plumb}" = "yes" -a "${do_unplumb}" = "yes" ] ; then
    make_log "*** Cannot mix plumb and unplumb in one operation!"
    exit 2
fi
if [ "${do_plumb}" = "no" -a "${do_unplumb}" = "no" ] ; then
    make_log "*** Must specify either plumb or unplumb."
    exit 2
fi

if [ -z "${service}" ] ; then
    make_log "*** Service name must be specified."
    exit 2
fi

if [ -z "${vip}" ] ; then
    make_log "*** VIP must be specified."
    exit 2
fi

if [ -z "${device}" ] ; then
    make_log "*** Device must be specified."
    exit 2
fi

if [ ! -x ${SVCTEST} ] ; then
    chmod +x ${SVCTEST}
fi

${SVCTEST} ${service} >/dev/null 2>&1
is_service=$?

if [ "${is_service}" -ne 0 ] ; then
    make_log "Service ${service} not running on this node."
    exit 1
fi

if [ "${do_plumb}" = "yes" ] ; then
     remove_interface
     make_log "ifconfig ${device} addif ${vip} ${netmask} up"
     ifconfig ${device} addif ${vip} ${netmask} up >>${LOGFILE} 2>&1
else
     remove_interface
fi

