#!/bin/sh
# Script:        package.sh
#
# Author:        High-Availability.Com Ltd
#
# Platform:      UNIX
#
# Description:   Functions for postinstall and preremove package install scripts.
#

BASEDIR=${BASEDIR:-/}
export BASEDIR

if [ "${INCLUDED_PACKAGE_SH}X" = "X" ] ; then
INCLUDED_PACKAGE_SH=done
export INCLUDED_PACKAGE_SH

#
# Function: config_db_defaults()
# Description: Creates a configuration database and populates it with default parameter
#              values.
# Args: none
#
config_db_defaults()
{
    install_log "config_db_defaults: called"
    console_log "Creating database with default properties"

    if [ -x "${BASEDIR}${CDB_EXEC}" ] ; then
        install_log "config_db_defaults: creating/populating configuration database"

        config_db_set prop_abort_unfenced_import                    create "${PROP_ABORT_UNFENCED_IMPORT__VALUE}"                "[boolean] Prevents an unfenced pool being imported. When set to true, a pool in which no disk is persistently reserved will not be imported and the service set to broken_safe should an import be attempted."
        config_db_set prop_additional_common_directories            create "${PROP_ADDITIONAL_COMMON_DIRECTORIES__VALUE}"        "[string] Specifies a colon separated list of extra directories to search when a common (shared) startup directory is specified for a service in the service configuration. These directories will be searched to build up a list of scripts/programmes to execute when starting/stopping a service. Duplicate elements (i.e. those with the same file name found in more than one common directory) will be selected and executed from the first instance found by prioritising the property directory list over the service configured directory. For example with both /usr/rc.appliance.c/S01announce and /opt/HAC/RSF-1/etc/rc.appliance.c/S01announce the S01announce script from /usr/rc.appliance.c will be the only one run at service start time."
        config_db_set prop_cluster_hook                             create "${PROP_CLUSTER_HOOK__VALUE}"                         "[string] A property that specifies an external command to be run during service start and stop. By default this property is emptyand therefore no action is taken. The properties use is intended to allow system integrators to build in their own start/stop hooks as required."
        config_db_set prop_comstar_standby_luns_before_failover     create "${PROP_COMSTAR_STANDBY_LUNS_BEFORE_FAILOVER__VALUE}" "[boolean] When set to true, COMSTAR LUNS will be placed into standby before failover/deletion."
        config_db_set prop_python_rpc_server_portno                 create "${PROP_PYTHON_RPC_SERVER_PORTNO__VALUE}"             "[numeric] Specifies the port number the RSF-1 Python RPC server listens on."
        config_db_set prop_scsi2_threaded_reserve_count             create "${PROP_SCSI2_THREADED_RESERVE_COUNT__VALUE}"         "[numeric] Specifies the maximum number of parallel SCSI-2 reserves to issue at any one time on a pool. The default value of -1 means there is no limit."
        config_db_set prop_use_zfs_cache                            create "${PROP_USE_ZFS_CACHE__VALUE}"                        "[boolean] When set to true, uses the ZFS cache file as part of the import to reduce the overall import time."
        config_db_set prop_event_notify                             create "${PROP_EVENT_NOTIFY__VALUE}"                         "[boolean] When set to true send cluster events (such as service start/stop) through the event notifier interface."
        config_db_set prop_mhdc_use_failfast                        create "${PROP_MHDC_USE_FAILFAST__VALUE}"                    "[boolean] When set to true SCSI-2/PGR3 disk reservations have fail fast protection applied. Losing a reservation results in a system panic (and thus cross-mounts are avoided)."
        config_db_set prop_mhdc_use_pgr3                            create "${PROP_MHDC_USE_PGR3__VALUE}"                        "[boolean] When set to true, disk reservations will use PGR3. When set to false SCSI-2 reservations will be used."
        config_db_set prop_scsi2_drive_count                        create "${PROP_SCSI2_DRIVE_COUNT__VALUE}"                    "[numeric] Determines the number of disks in the pool on which SCSI-2 reservations are applied."
        config_db_set prop_pgr3_drive_count                         create "${PROP_PGR3_DRIVE_COUNT__VALUE}"                     "[numeric] Determines the number of disks in the pool on which PGR3 reservations are applied."
        config_db_set prop_zpool_fail_mode                          create "${PROP_ZPOOL_FAIL_MODE__VALUE}"                      "[string] Determines the behaviour of a catastrophic pool failure due to a loss of device connectivity or the failure of all devices in the pool. Options are: panic, continue, wait - see ZFS fail mode property for more details."
        config_db_set prop_rsf_zfs_event_logging                    create "${PROP_RSF_ZFS_EVENT_LOGGING__VALUE}"                "[boolean] When set to true, ZFS sysevents are logged to the RSF-1 log file (current events are cache file changes - use this option to ensure cache file synchronisation is occurring)."
        config_db_set prop_skip_vip_check                           create "${PROP_SKIP_VIP_CHECK__VALUE}"                       "[boolean] When set to true, the initial VIP check done on service start is bypassed. Under normal circumstances this check should always be left on (i.e. the setting should be false)."
        config_db_set prop_skip_pgr3_log_and_cache                  create "${PROP_SKIP_PGR3_LOG_AND_CACHE__VALUE}"              "[boolean] When set to true, do not attempt to reserve the log and cache devices when using PGR3 reservations."
        config_db_set prop_scsi2_threaded_reserve                   create "${PROP_SCSI2_THREADED_RESERVE__VALUE}"               "[boolean] When set to true, SCSI-2 reservations are issued in parallel and as such the total reservation/release time during poll import/export is reduced. Normally a SCSI-2 reserve takes on average 2 seconds, so reserving for example 6 drives could take/add up to 12 seconds to the overall fail over time - switching to parallel reservations means all 6 drives can be reserved in under two seconds. Due to a known issue with SAS disks, this option is disabled by default."
        config_db_set prop_scsi2_retry_count                        create "${PROP_SCSI2_RETRY_COUNT__VALUE}"                    "[numeric] Sets the number of SCSI-2 TKOWN retry attempts when an I/O error is encountered back from the disk being operated on - set to 0 for no retries."
        config_db_set prop_res_drive_test_retries                   create "${PROP_RES_DRIVE_TEST_RETRIES__VALUE}"               "[numeric] Sets the number of retry attempts when the disk_list drive test fails in S14res_drives - set to 0 for no retries."
        config_db_set prop_res_drive_test_delay                     create "${PROP_RES_DRIVE_TEST_DELAY__VALUE}"                 "[numeric] Sets the delay time between retries of the disk_list drive test in S14res_drives"
        config_db_set prop_ssh_bound                                create "${PROP_SSH_BOUND__VALUE}"                            "[boolean] Set to true when cluster nodes are bound together, otherwise false and no scp or ssh attempted."
        config_db_set prop_gui_failover_state_sticky                create "${PROP_GUI_FAILOVER_STATE_STICKY__VALUE}"            "[boolean] When moving a service move over the service state (automatic/manual) with it."
        config_db_set prop_fping_retries                            create "${PROP_FPING_RETRIES__VALUE}"                        "[numeric] Number of fping retries when performing VIP check prior to service startup. A value of 0 or -1 means just the one check, a numeric value means check this number of times if and only if the VIP is pingable on the first check. Ignored if the VIP is not detected before service startup."
        config_db_set prop_pool_hook                                create "${PROP_POOL_HOOK__VALUE}"                            "[string] A property specifying an external command to be run during pool import by S20zfs (and subsequent export by K80zfs). By default this property is empty and therefore no action is taken.  The properties use is intended to allow system integrators to build in their own import/export hooks as required."
        config_db_set prop_res_drive_wait                           create "${PROP_RES_DRIVE_WAIT__VALUE}"                       "[numeric] A property specifying an additional delay introduced before removing the reservation (required for non-persistent reservations. Default value is 0."
        config_db_set prop_zpool_sync_cache                         create "${PROP_ZPOOL_SYNC_CACHE__VALUE}"                     "[boolean] Synchronize ZFS cache files between HA nodes whenever a ZPOOL cache is updated. The copy is instigated by a sysevent listener."
        config_db_set prop_zpool_threads_number                     create "${PROP_ZPOOL_THREADS_NUMBER__VALUE}"                 "[numeric] Number of threads to run when option prop_zpool_threads_option_t is set to true."
        config_db_set prop_zpool_dtrace_needed_timeout_seconds      create "${PROP_ZPOOL_DTRACE_NEEDED_TIMEOUT_SECONDS__VALUE}"  "[numeric] Threshold (in seconds) after which point the dtrace import/export scripts should be enabled"

        #
        # These properties are used by the REST API
        #
        config_db_set privPort                                      create 4330                                                  "[numeric] Port to use as the private port for the REST API"
        config_db_set pubPort                                       create 4330                                                  "[numeric] Port to use as the public port for the REST API"
        config_db_set cookieExpireTime                              create 10800                                                 "[numeric] Cookie expiry timeout (in seconds) for the REST API"
        config_db_set shortTimeout                                  create 4                                                     "[numeric] Short timeout (in seconds) for the REST API"
        config_db_set regTimeout                                    create 10                                                     "[numeric] Regular timeout (in seconds) for the REST API"
        config_db_set longTimeout                                   create 40                                                    "[numeric] Long timeout (in seconds) for the REST API"
        config_db_set verylongTimeout                               create 60                                                    "[numeric] Very long timeout (in seconds) for the REST API"
        config_db_set veryverylongTimeout                           create 120                                                   "[numeric] Very very long timeout (in seconds) for the REST API"
        config_db_set useACL                                        create true                                                  "[boolean] Use an Access Control List (ACL) for REST communications"
        config_db_set useSSL                                        create true                                                  "[boolean] Use SSL encryption for REST API communications"
        config_db_set logLevel                                      create "INFO"                                                "[string] Log level to use for the REST API log file"

        #
        # These properties are used by the GUI are are therefore not currently required to be exported
        # as environment variables in rsf.sh
        config_db_set prop_gui_enable_ipv6                          create "${PROP_GUI_ENABLE_IPV6__VALUE}"                      "[boolean] When set to true enables IPV6 support for Cluster VIPs along with IPV4 in the Cluster GUI."
        config_db_set prop_gui_res_drive_support                    create "${PROP_GUI_RES_DRIVE_SUPPORT__VALUE}"                "[boolean] When set to true enables reservation drive selection in the GUI rather than automatic generation. This ability in the GUI is being deprecated in favour of automatic drive selection."
        config_db_set prop_gui_enable_fastfail                      create "${PROP_GUI_ENABLE_FASTFAIL__VALUE}"                  "[boolean] When set to true enable the fast failover option in the GUI - reserved for future use."
        config_db_set prop_gui_javascript_debug                     create "${PROP_GUI_JAVASCRIPT_DEBUG__VALUE}"                 "[boolean] When set to true debugging messages are written to the browsers JavaScript console (when active). This option is not supported on IE browsers due to a bug in their JavaScript implementation."
        config_db_set prop_gui_serial_hb_support                    create "${PROP_GUI_SERIAL_HB_SUPPORT__VALUE}"                "[boolean] When set to true allows the user to configure serial heartbeats in the cluster from the GUI."
        config_db_set prop_gui_enable_comstar_browser               create "${PROP_GUI_ENABLE_COMSTAR_BROWSER__VALUE}"           "[boolean] When set to true reveals the COMSTAR browser GUI component."
        config_db_set prop_gui_enable_diagnostics                   create "${PROP_GUI_ENABLE_DIAGNOSTICS__VALUE}"               "[boolean] When set to true reveals the diagnostics GUI component."
        config_db_set prop_gui_diagnostics_test                     create "${PROP_GUI_DIAGNOSTICS_TEST__VALUE}"                 "[boolean] Used for testing purposes only."
        config_db_set prop_gui_cluster_initialized                  create "${PROP_GUI_CLUSTER_INITIALIZED__VALUE}"              "[boolean] Set to true once the cluster has been initialized. This is a GUI initialization speedup that signifies a number of (one off) GUI checks have completed successfully and therefore do not need to be run again."
        config_db_set prop_gui_disable_online_help                  create "${PROP_GUI_DISABLE_ONLINE_HELP__VALUE}"              "[boolean] Enables/Disables the showing of the About, FAQ menu items and manual page links in the title bar of each cluster panel."
        #
        # Now create some configuration file constant.
        config_db_set prop_gui_default_inittimeout                  create "${PROP_GUI_DEFAULT_INITTIMEOUT__VALUE}"              "[numeric] The default initial timeout value for the cluster."
        config_db_set prop_gui_default_runtimeout                   create "${PROP_GUI_DEFAULT_RUNTIMEOUT__VALUE}"               "[numeric] The default run timeout value for the cluster."
        config_db_set prop_gui_default_cluster_name                 create "${PROP_GUI_DEFAULT_CLUSTER_NAME__VALUE}"             "[string] The default name for a new cluster."
        config_db_set prop_gui_plugin_version                       create "${PROP_GUI_PLUGIN_VERSION__VALUE}"                   "[string] Identifies the plugin version.."
        #
        # Delete some old values not used any more.
        config_db_set zpool_sync_cache                              delete
        config_db_set prop_no_zfs_cache                             delete
        config_db_set no_zfs_cache                                  delete
        config_db_set prop_mhdc_stec_skip                           delete
        #
        # Changes made to settings
        if [ "${COMPANY_OS}" = "linux" -o "${COMPANY_OS}" = "freebsd" ]; then
            config_db_set prop_mhdc_use_pgr3                        update ${BOOL_STR_TRUE}
            config_db_set prop_scsi2_drive_count                    update 0
            config_db_set prop_pgr3_drive_count                     update -1
        fi
        #
        # Always update this one with the latest version string.
        config_db_set prop_gui_plugin_version                       update "${PROP_GUI_PLUGIN_VERSION__VALUE}"

        if is_nexenta; then
            install_log "config_db_defaults: Nexenta platform detected, applying specific props"
            if is_nexenta_4 || is_nef; then
                config_db_set prop_zpool_dtrace_import_export           create ${BOOL_STR_TRUE}
                config_db_set prop_zpool_import_option_m                create ${BOOL_STR_TRUE}
                config_db_set prop_zpool_export_option_c                create ${BOOL_STR_TRUE}
                config_db_set prop_zpool_threads_option_t               create ${BOOL_STR_TRUE}
                config_db_set prop_nfs_failover_support                 create ${BOOL_STR_TRUE}
            else
                config_db_set prop_zpool_dtrace_import_export           create ${BOOL_STR_FALSE}
                config_db_set prop_zpool_import_option_m                create ${BOOL_STR_FALSE}
                config_db_set prop_zpool_export_option_c                create ${BOOL_STR_FALSE}
                config_db_set prop_zpool_threads_option_t               create ${BOOL_STR_FALSE}
            fi
            if is_nexenta_3 || is_nexenta_4; then
                config_db_set prop_ssh_bound                            update ${BOOL_STR_TRUE}
            else
                config_db_set prop_ssh_bound                            update ${BOOL_STR_FALSE}
                config_db_set prop_disable_site_safe                    create "${PROP_DISABLE_SITE_SAFE__VALUE}" "[boolean] When set to true the cluster will never use broken_site_safe. Only use this mode if you are 100% sure there will not be a site partition"
                config_db_set prop_disable_drive_monitor                create "${PROP_DISABLE_DRIVE_MONITOR__VALUE}" "[boolean] When set to true the cluster will not continually monitor the status of reservation and heartbeat drives"
            fi
            if is_nexenta_4; then
                config_db_set prop_gui_enable_fastfail                  update ${BOOL_STR_FALSE}
                config_db_set prop_gui_enable_ipv6                      update ${BOOL_STR_FALSE}
            fi
        else
            config_db_set prop_zpool_dtrace_import_export               create ${BOOL_STR_FALSE}
            config_db_set prop_zpool_import_option_m                    create ${BOOL_STR_FALSE}
            config_db_set prop_zpool_export_option_c                    create ${BOOL_STR_FALSE}
            config_db_set prop_zpool_threads_option_t                   create ${BOOL_STR_FALSE}
        fi

        config_db_set prop_comstar_support                          create ${BOOL_STR_TRUE}

        config_db_set prop_comstar_support                          desc   "[boolean] Enables failover support for COMSTAR targets configured on clustered volumes."
        config_db_set prop_zpool_dtrace_import_export               desc   "[boolean] Enable the running of custom dtrace scripts to trace ZPOOL import/export."
        config_db_set prop_zpool_import_option_m                    desc   "[boolean] When set to true, zpool import supports the -m (import missing) option."
        config_db_set prop_zpool_export_option_c                    desc   "[boolean] When set to true, zpool export supports the -c (flush cache) option."
        config_db_set prop_zpool_threads_option_t                   desc   "[boolean] When set to true, enables parallel import/export pool mounts using threads."
    else
        install_log "config_db_defaults: could not find prog ${BASEDIR}${CDB_EXEC}"
    fi

    install_log "config_db_defaults: returning"

    return 0
}

#
# Function: install_log
# Description: save installation information to a log file
# Args:
#      anything wanting logging
#
INSTALL_LOG="${BASEDIR}opt/HAC/RSF-1/log/.hac-install-$$.log"
install_log()
{
    echo "[`date +'%b %d %H:%M:%S'`] $*" >> ${INSTALL_LOG}
}

console_log()
{
    if [ "$1" = "--append" ]; then
        shift
        TEXT=${TEXT}$*
    else
        TEXT="$*"
    fi
    [ -z "${console_log_len}" ] && console_log_len=0 && printf "\n"
    printf "\r%${console_log_len}s\r" " "
    console_log_len=${#TEXT}
    printf "%s" "${TEXT}"
    install_log "${TEXT}"
}

#
# Function:    boot_link()
# Reloc:       complete
# Description: Front end OS generic function to install or remove rc boot files
# Args:
#              1) -e for enable, -d for disable
#
boot_link()
{
    install_log "boot_link: called arg: $1"

    MAKE_LINK_ARG="Y"

    while [ $# -gt 0 ]; do
    case "$1" in
        -e) MAKE_LINK_ARG="Y"
            ACTION="Enable"
        ;;

        -d) MAKE_LINK_ARG="N"
            ACTION="Disable"
        ;;

        -*) echo " boot_link: called with unknown option...${1}"
        ;;
    esac
    shift
    done

    console_log "${ACTION} RSF-1 on boot"

    # work out rc dir paths
    WARN=N
    install_log " boot_link: making links for ${COMPANY_OS}"
    case "${COMPANY_OS}" in
    'sunos')
        make_links N etc etc 2 0 1 S
        ;;

    'linux')
        if which systemctl >/dev/null 2>&1 && [ -f ${PRODUCT_DIR}/lib/rsf-1.service ]; then
            case ${MAKE_LINK_ARG} in
            Y)
                cp ${PRODUCT_DIR}/lib/rsf-1.service /lib/systemd/system/rsf-1.service
                systemctl enable rsf-1
                ;;
            N)
                systemctl disable rsf-1
                rm -f /lib/systemd/system/rsf-1.service
                ;;
            esac >> ${INSTALL_LOG} 2>&1
        else
            # Red Hat only
            # Slight mod here to check for init.d as well - SuSE has /etc/rc.d and /etc/init.d
            # but no /etc/rc.d/init.d
            # 28 July 2009: added a default start level as some distros (Ubuntu) do not have inittab.
            #
            START_LEVEL=2
            if [ -f /etc/inittab ] ; then
            START_LEVEL=`awk 'BEGIN{FS=":"} $3 ~ /initdefault/ { print $2; }' /etc/inittab` # work out default run level
            fi

            if [ -d "/etc/rc.d" -a -d "/etc/rc.d/init.d" ]; then    # Red Hat
            make_links ${MAKE_LINK_ARG} etc/rc.d etc/rc.d ${START_LEVEL} 0 6
            elif [ -d "/etc/init.d" ]; then # Debian, SuSE, others
            if [ -d "/etc/init.d/rc${START_LEVEL}.d" ] ; then
                make_links ${MAKE_LINK_ARG} etc etc/init.d ${START_LEVEL} 0 6 # SuSE
            else
                make_links ${MAKE_LINK_ARG} etc etc ${START_LEVEL} 0 6
            fi
            else
            WARN=Y
            fi
        fi
        ;;

    'freebsd')
        RC_SOURCE=/opt/HAC/RSF-1/init.d
        RC_DEST=/etc/rc.d
        RC_LOCAL_DEST=/usr/local/etc/rc.d
        RSF_STARTUP_SCRIPT=/etc/rc.d/rsf
        STARTUP_CONF=/etc/rc.conf

        if [ ! -d ${RC_LOCAL_DEST} ] ; then
            mkdir ${RC_LOCAL_DEST}
            chmod 755 ${RC_LOCAL_DEST}
        fi

        if [ "${MAKE_LINK_ARG}" = "Y" ] ; then
            cp ${RC_SOURCE}/rc-rsf ${RC_LOCAL_DEST}/rsf
            cp ${RC_SOURCE}/rc-restapi ${RC_LOCAL_DEST}/rsfrestapi
            chmod +x ${RC_LOCAL_DEST}/rsf ${RC_LOCAL_DEST}/rsfrestapi
            grep '^rsf_enable=' ${STARTUP_CONF} >/dev/null 2>&1 || echo 'rsf_enable="YES"' >> ${STARTUP_CONF}
            grep '^rsfrestapi_enable=' ${STARTUP_CONF} >/dev/null 2>&1 || echo 'rsfrestapi_enable="YES"' >> ${STARTUP_CONF}
        else
            service rsfrestapi stop
            service rsfwebapp stop
            rm -f ${RC_LOCAL_DEST}/rsf ${RC_LOCAL_DEST}/rsfrestapi ${RC_LOCAL_DEST}/rsfwebapp 2>/dev/null
            sed -e '/^rsf_enable=/d' -e '/^rsfrestapi_enable=/d' -e '/^rsfwebapp_enable=/d' ${STARTUP_CONF} > /tmp/${STARTUP_CONF##*/}
            mv -f /tmp/${STARTUP_CONF##*/} ${STARTUP_CONF}
        fi
        ;;

    *)
        #
        # We don't support this yet - warn the user.
        install_log " The OS ${COMPANY_OS} is not supported in boot_link and therefore nothing has"
        install_log " been done to link the rsfmonrc file to the /etc/init.d style directory."
        install_log " Please manually create the link."
        WARN=Y
        ;;
    esac

    if [ "${WARN}" = "Y" ] ; then
    echo "Warning: boot enabling ${PRODUCT} on ${COMPANY_OS} is not supported - please manually"
    echo "Warning: syslink ${PRODUCT_DIR}/init.d/${PRODUCT_BINARY}rc into your system rc directories."
    fi

    install_log "boot_link: returning"
}

#
# Function:    make_links()
# Reloc:       complete
# Description: Link the start/stop directory scripts
# Args:
#              1) if Y then install links, else remove
#              2) where init.d on this architecture lives (if /sbin then place in /sbin/init.d/)
#              3) where the rc*.d directories live. Usually same place as init.d, but some OS's (SuSE 9.0 for example)
#                 have the rc*.d directories in the /etc/init.d directory.
#              4) the rc directory number to link the start on boot rc script
#              *) the rc directory(s) number(s) to link the stop on shutdown rc script
#
make_links()
{
    install_log "make_links: called"

    ENABLE=$1
    shift
    MASTER_ETC_DIR=${BASEDIR}${1}
    shift
    LINK_ETC_DIR=${BASEDIR}${1}
    shift
    START_LEVEL_DIR=$1
    shift
    STOP_LEVEL_DIRS=$*

    #
    # Set the RC directory numbering based on the OS.
    case "${COMPANY_OS}" in
    'linux')
        PRODUCT_RC_START=15
        PRODUCT_RC_STOP=85
        ;;
    *)
        PRODUCT_RC_START=99
        PRODUCT_RC_STOP=1
        ;;
    esac

    #
    # Insert any leading zero's we'll need
    if [ ${PRODUCT_RC_START} -lt 10 ] ; then
    PRODUCT_RC_START="0${PRODUCT_RC_START}"
    fi

    if [ ${PRODUCT_RC_STOP} -lt 10 ] ; then
    PRODUCT_RC_STOP="0${PRODUCT_RC_STOP}"
    fi

    MASTER_RC="${MASTER_ETC_DIR}/init.d/${PRODUCT_BINARY}rc"

    if [ "${ENABLE}" = "Y" ] ; then
    echo " "
    console_log "Enabling ${PRODUCT} on boot..."

        install_log " make_links: linking ${PRODUCT_RC} to ${MASTER_RC}"
        if [ -f "${BASEDIR}${MASTER_RC}" ] ; then
            rm "${BASEDIR}${MASTER_RC}"
        fi
    ln -s "${PRODUCT_RC}" "${BASEDIR}${MASTER_RC}"

        install_log " make_links: linking ${MASTER_RC} to ${LINK_ETC_DIR}/rc${START_LEVEL_DIR}.d/S${PRODUCT_RC_START}${PRODUCT_LOWCASE}"
        if [ -f "${LINK_ETC_DIR}/rc${START_LEVEL_DIR}.d/S${PRODUCT_RC_START}${PRODUCT_LOWCASE}" ] ; then
            rm "${LINK_ETC_DIR}/rc${START_LEVEL_DIR}.d/S${PRODUCT_RC_START}${PRODUCT_LOWCASE}"
        fi
    ln -s "${MASTER_RC}" "${LINK_ETC_DIR}/rc${START_LEVEL_DIR}.d/S${PRODUCT_RC_START}${PRODUCT_LOWCASE}"

    for var_stop in ${STOP_LEVEL_DIRS}; do
            install_log " make_links: linking ${MASTER_RC} to ${LINK_ETC_DIR}/rc${var_stop}.d/K${PRODUCT_RC_STOP}${PRODUCT_LOWCASE}"
            if [ -f "${LINK_ETC_DIR}/rc${var_stop}.d/K${PRODUCT_RC_STOP}${PRODUCT_LOWCASE}" ] ; then
                rm -f "${LINK_ETC_DIR}/rc${var_stop}.d/K${PRODUCT_RC_STOP}${PRODUCT_LOWCASE}"
            fi
        ln -s "${MASTER_RC}" "${LINK_ETC_DIR}/rc${var_stop}.d/K${PRODUCT_RC_STOP}${PRODUCT_LOWCASE}"
    done
    else
    install_log " make_links: disabling ${PRODUCT} on boot..."

        install_log " make_links: removing ${MASTER_RC}"
    rm -f "${MASTER_RC}" >/dev/null 2>&1

        install_log " make_links: removing ${LINK_ETC_DIR}/rc${START_LEVEL_DIR}.d/S${PRODUCT_RC_START}${PRODUCT_LOWCASE}"
    rm -f "${LINK_ETC_DIR}/rc${START_LEVEL_DIR}.d/S${PRODUCT_RC_START}${PRODUCT_LOWCASE}" >/dev/null 2>&1

    for var_stop in ${STOP_LEVEL_DIRS}; do
            install_log " make_links: removing ${LINK_ETC_DIR}/rc${var_stop}.d/K${PRODUCT_RC_STOP}${PRODUCT_LOWCASE}"
        rm -f "${LINK_ETC_DIR}/rc${var_stop}.d/K${PRODUCT_RC_STOP}${PRODUCT_LOWCASE}" >/dev/null 2>&1
    done
    fi

    install_log "make_links: returning"
}

#
# Function:    add_rsf_services()
# Reloc:       complete
# Description: Add RSF entries to /etc/services file
# Args:
#              none
#
add_rsf_services()
{
    install_log "add_rsf_services called"

    console_log "Adding ${PRODUCT} to services file..."

    add_to_services rsf-1 1195 high-availability.com

    install_log "add_rsf_services returning"
}

#
# Function:    add_to_services()
# Reloc:       complete
# Description: Add entries to services file
#              Our official port is 1195.
#              Request on tcp, heartbeat on udp
# Args:
#              1) Service name (rsfnet, rsfreq)
#              2) Service port (6969, 6970)
#              *) String to include in comment
# Possible extensions:
#      If required add -u and -t flags for udp/tcp entry decisions
#
add_to_services()
{
    install_log "add_to_services: called"
    SERVICES_FILE="${BASEDIR}etc/services"

    if [ $# -lt 3 ] ; then
    echo " Error: not enough args to add_to_services"
    echo " Usage: add_to_services name port comment"
    echo "Called: add_to_services $*"
    echo
    return
    fi

    SERVICENAME=$1
    shift
    SERVICEPORT=$1
    shift
    SERVICETYPE="$*"

    grep ${SERVICENAME} ${SERVICES_FILE} >/dev/null 2>&1

    if [ $? -eq 1 ] ; then
    install_log " add_to_services: adding ${SERVICENAME} ${SERVICEPORT} to ${SERVICES_FILE}"
    grep " ${SERVICEPORT}" ${SERVICES_FILE} | grep -v '^#' >/dev/null 2>&1 # check nothing else is using ${SERVICEPORT}
    if [ $? -eq 1 ] ; then
        echo "${SERVICENAME}        ${SERVICEPORT}/tcp          # RSF-1 clustering TCP ${SERVICETYPE} assigned by www.iana.org" >> ${SERVICES_FILE}
        echo "${SERVICENAME}        ${SERVICEPORT}/udp          # RSF-1 clustering UDP ${SERVICETYPE} assigned by www.iana.org" >> ${SERVICES_FILE}
    else
        install_log " add_to_services: ${SERVICENAME} entries not added to ${SERVICES_FILE} - ${SERVICEPORT} already in use"
        echo
        echo "${SERVICENAME} entries not added to ${SERVICES_FILE} - ${SERVICEPORT} already in use:"
        grep ${SERVICEPORT} ${SERVICES_FILE}
        echo
    fi
    else
        install_log " add_to_services: not adding ${SERVICENAME} ${SERVICEPORT} to ${SERVICES_FILE} - entry already exists"
        grep ${SERVICEPORT} ${SERVICES_FILE} >> ${INSTALL_LOG}
    fi

    install_log "add_to_services: returning"
}

#
# Function:    create_rsf_account()
# Reloc:       complete
# Description: Add _rsfadmin account to rsfpasswd file.
# Args:
#              $1 - account to add
#
create_rsf_account()
{
    PASSWORDFILE="${BASEDIR}opt/HAC/RSF-1/etc/passwd"
    console_log "Creating rsf account"

    ADMIN_NAME=$1
    _PASSWD_=$2

    ADD="false"

    if [ -f ${PASSWORDFILE} ] ; then
    grep ${ADMIN_NAME} ${PASSWORDFILE} >/dev/null 2>&1
    #
    # Grep tells us that the ID does not exist, so add it.
    if [ $? -ne 0 ] ; then
        ADD="true"
    fi
    else # file does not exist - so add anyway.
    ADD="true"
    fi

    if [ "${ADD}" = "true" ] ; then
    install_log " create_rsf_account: adding ${ADMIN_NAME} to ${PASSWORDFILE}"
    echo "${ADMIN_NAME}:${_PASSWD_}" >> ${PASSWORDFILE}
    fi

    install_log "create_rsf_account: returning"
}

#
# Function:    create_rsf_default_config()
# Reloc:       complete
# Description: Create a config file if one does not exists. Creates a simple one that in effect means RSF-1 is listening for updates etc.
# Args:        none
#
create_rsf_default_config()
{
    install_log "create_rsf_default_config: called"
    console_log "Creating default config"

    #
    # Added new variable "BOOTSTRAP_CONFIG_FILE" to replace "BOOTSTRAP"
    # as it's more descriptive when it's used further on in the config.
    #
    # Build it anyway
    CONFIGFILE="${BASEDIR}opt/HAC/RSF-1/etc/config"

    if [ -f "${CONFIGFILE}" ] ; then
        install_log " create_rsf_default_config: Config file ${CONFIGFILE} already exists"
        CONFIG_SUM="`sum < ${CONFIGFILE}`"
        RECONF_SUM="`sum < ${CONFIGFILE}.reconfigure`"
        if [ -n "${CONFIG_SUM}" -a "${CONFIG_SUM}" = "${RECONF_SUM}" ]; then
            install_log " create_rsf_default_config: Existing config file is bootstrap config"
            BOOTSTRAP_CONFIG_FILE="true"
        else
            BOOTSTRAP_CONFIG_FILE="false"
        fi
    else
        install_log " create_rsf_default_config: installing ${CONFIGFILE}.reconfigure as ${CONFIGFILE}"
        BOOTSTRAP_CONFIG_FILE="true"
        cp ${CONFIGFILE}.reconfigure ${CONFIGFILE}
    fi

    install_log "create_rsf_default_config: returning"
}

#
# Function:    clear_zfs_cache()
# Reloc:       complete
# Description: Remove the saved ZFS cache between updates
# Args:        none
#
clear_zfs_cache()
{
    VOL_CACHE="${BASEDIR}opt/HAC/RSF-1/etc/volume-cache"

    install_log "clear_zfs_cache: called"
    console_log "Clear ZFS cache"

    if [ -d "${VOL_CACHE}" ] ; then
    install_log "clear_zfs_cache: clearing ZFS device cache in ${VOL_CACHE}"
    rm -rf ${VOL_CACHE} && mkdir ${VOL_CACHE}
    fi

    install_log "clear_zfs_cache: returning"
}

#
# Function:    link_mapmgr()
# Reloc:       complete
# Description: If we have a copy of mapmgr in /usr/bin/mapmgr and one in
#              /opt/HAC/RSF-1/bin/mapmgr then move out the older mapmgr and
#              link it to the RSF-1 delivered one.
# Args:        none
link_mapmgr()
{
    install_log "link_mapmgr: called"
    USR_BIN_MAPMGR="${BASEDIR}usr/bin/mapmgr"
    OPT_HAC_MAPMGR="/opt/HAC/RSF-1/bin/mapmgr"
    console_log "Creating mapmgr links"

    if [ -f "${USR_BIN_MAPMGR}" -a -f "${BASEDIR}${OPT_HAC_MAPMGR}" ] ; then
        if [ ! -h "${USR_BIN_MAPMGR}" ] ; then
            if [ ! -f "${USR_BIN_MAPMGR}.pre-rsf-1" ] ; then
                install_log " link_mapmgr: creating link from ${OPT_HAC_MAPMGR} to ${USR_BIN_MAPMGR}"
                mv "${USR_BIN_MAPMGR}" "${USR_BIN_MAPMGR}.pre-rsf-1"
                ln -s "${OPT_HAC_MAPMGR}" "${USR_BIN_MAPMGR}"
            else
                install_log " link_mapmgr: ${USR_BIN_MAPMGR}.pre-rsf-1 exists - no action (possible confused state)"
            fi
        else
            install_log " link_mapmgr: ${USR_BIN_MAPMGR} is already a symlink"
        fi
    else
        install_log " link_mapmgr: one of ${USR_BIN_MAPMGR} or ${OPT_HAC_MAPMGR} or ${BASEDIR}${OPT_HAC_MAPMGR} does not exist"
    fi

    install_log "link_mapmgr: returning"
}

#
# Function:    restart_rsf()
# Reloc:       complete
# Description: At the end of the postinstall script, restart RSF-1.
#              Currently just done for NexentaOS.
# Args:
#              none
#
restart_rsf()
{
    install_log "restart_rsf: called"

    if [ "${OSTYPE}" = "FreeBSD" ]; then
        install_log " restart_rsf: FreeBSD detected - not starting rsfmon automatically"
        start_rsf=0
    elif [ "${BOOTSTRAP_CONFIG_FILE}" = "true" ]; then
        start_rsf=1
    else
	#
        # Not BOOTSTRAP_CONFIG_FILE, so we don't want to do rsfctl start
        # - could be running services with a killed rsfmon
        install_log " restart_rsf: Config file in place - not starting rsfmon"
        start_rsf=0
    fi

    if [ ${start_rsf} -eq 1 ]; then
        console_log "Restarting RSF-1"
        if [ -z "${BASEDIR}" -o "${BASEDIR}" = "/" ] ; then
            if [ -x ${SVCADM} ]; then
                install_log " restart_rsf: running svcadm enable rsf-1"
                echo
                ${SVCADM} enable -s rsf-1
                echo
            elif which systemctl >/dev/null 2>&1; then
                install_log " restart_rsf: running systemctl start rsf-1"
                systemctl start rsf-1
            else
                install_log " restart_rsf: running /opt/HAC/RSF-1/init.d/rsfmonrc start"
                /opt/HAC/RSF-1/init.d/rsfmonrc start
            fi
        else
            install_log " restart_rsf: BASEDIR is set - not starting rsfmon"
        fi
    else
        install_log " restart_rsf: not necessary to restart RSF-1"
    fi

    install_log "restart_rsf: returning"
}

#
# Function:    stop_rsf
# Reloc:       complete
# Description: Shuts down the running copy of RSF-1
# Args:        none
#
# OS_SPECIFIC
#
stop_rsf()
{
    console_log "Stopping RSF-1"

    case "${COMPANY_OS}" in
    'sunos')
        console_log "Stopping RSF-1 (`basename ${SVCADM}` disable rsf-1)"
        echo
        ${SVCADM} disable -s rsf-1
        echo
        ;;
    'linux')
        if which systemctl >/dev/null 2>&1; then
            console_log "Stopping RSF-1 (systemctl stop rsf-1)"
            systemctl stop rsf-1
        else
            console_log "Stopping RSF-1 (`basename ${PRODUCT_RC}` stop)"
            (/bin/sh ${PRODUCT_RC} stop)
        fi
        ;;

    *)
        console_log "Stopping RSF-1 (`basename ${PRODUCT_RC}` stop)"
        (/bin/sh ${PRODUCT_RC} stop)
        ;;
    esac

    install_log "RSF-1 stopped"
}

#
# Function:    remove_rsf_services()
# Reloc:       complete
# Description: Remove rsf entries from /etc/services file
# Args: none
#
remove_rsf_services()
{
    SERVICES="${BASEDIR}etc/services"
    install_log "remove_rsf_services: called"
    console_log "Removing services from ${SERVICES}"

    remove_from_file ${SERVICES} rsfnet
    remove_from_file ${SERVICES} rsfreq
    remove_from_file ${SERVICES} rsf-1

    install_log "remove_rsf_services: returning"
}

#
# Function:    remove_from_file()
# Reloc:       complete
# Description: Remove lines added by HAC postinstall product files from system files.
# Args:
#       1) the file to remove from
#       2) the entry to remove
#
remove_from_file()
{
    install_log "remove_from_file: called"

    if [ $# -ne 2 ] ; then
    echo "Error: not enough args to remove_from_file"
    return
    fi

    install_log " remove_from_file: removing $2 from $1"

    FILETOREMOVEFROM=$1
    ENTRYTOREMOVE=$2

    if [ ! -f ${FILETOREMOVEFROM} ] ; then
    echo "Warning: file ${FILETOREMOVEFROM} does not exist!"
    return
    fi

    FILETOREMOVEFROMBCK=${FILETOREMOVEFROM}.bck          # Temporary backup copy of file we're removing from
    TMPFILE=/tmp/rff.$$                                  # Copy of file we work on
    TMPFILENEW=/tmp/rffnew.$$                            # Copy of file we work on with entries removed
    LINES=`wc -l ${FILETOREMOVEFROM} | awk '{print $1}'` # Count the number of lines in the file we're removing from
    LINESAFTER=`grep -v ${ENTRYTOREMOVE} ${FILETOREMOVEFROM}| \
        wc -l  | awk '{print $1}'`               # Count of what lines should be afer removing the entry

    rm -f "${TMPFILE}" "${TMPFILENEW}" >/dev/null 2>&1
    cp "${FILETOREMOVEFROM}" "${TMPFILE}" >/dev/null 2>&1

    if [ $? -eq 0 ] ; then # make sure it copies OK
    grep -v "${ENTRYTOREMOVE}" "${TMPFILE}" > "${TMPFILENEW}"
    NEWLINES=`wc -l ${TMPFILENEW} | awk '{print $1}'`
    if [ ${NEWLINES} -eq ${LINESAFTER} ] ; then
        mv "${FILETOREMOVEFROM}" "${FILETOREMOVEFROMBCK}" >/dev/null 2>&1
        cp "${TMPFILENEW}" "${FILETOREMOVEFROM}" >/dev/null 2>&1
        if [ $? -ne 0 ] ; then
        mv "${FILETOREMOVEFROMBCK}" "${FILETOREMOVEFROM}" >/dev/null 2>&1
        else
        rm -f "${FILETOREMOVEFROMBCK}" >/dev/null 2>&1
        fi
    fi
    fi
    rm -f "${TMPFILE}" "${TMPFILENEW}" >/dev/null 2>&1

    install_log "remove_from_file: returning"
}

#
# Function:    unlink_mapmgr()
# Reloc:       complete
# Description: rollback mapmgr to the one prior to the rsf-1 install
#              if the old state files exist
# Args: none
unlink_mapmgr()
{
    install_log "unlink_mapmgr: called"
    USR_BIN_MAPMGR="${BASEDIR}usr/bin/mapmgr"
    console_log "Removing mapmgr links"

    if [ -h "${USR_BIN_MAPMGR}" -a -f "${USR_BIN_MAPMGR}.pre-rsf-1"  ] ; then
        install_log "unlink_mapmgr: rolling back mapmgr from ${USR_BIN_MAPMGR}.pre-rsf-1 to ${USR_BIN_MAPMGR}"
        rm "${USR_BIN_MAPMGR}"
        mv "${USR_BIN_MAPMGR}.pre-rsf-1" "${USR_BIN_MAPMGR}"
    else
        if [ ! -h "${USR_BIN_MAPMGR}" ] ; then
            install_log "unlink_mapmgr: ${USR_BIN_MAPMGR} is not a symlink - no rollback action required"
        else
            install_log "unlink_mapmgr: ${USR_BIN_MAPMGR}.pre-rsf-1 does not exist - no rollback action required"
        fi
    fi

    install_log "unlink_mapmgr: returning"
}


#
# Function:    install_patches()
# Description: Install any patches that have not already been installed
# Args:        none
#
install_patches()
{
    console_log "Installing patches"
    for patch_file in `ls ${BASEDIR}${PATCH_DIR} | grep -v "^\."`; do
    patch_file=${BASEDIR}${PATCH_DIR}/${patch_file}
        PATCH=${patch_file##*/}
        if ! [ -f ${BASEDIR}${PATCH_RECORD_DIR}/${PATCH} ]; then
            install_log "Installing patch ${PATCH}"
            install_log "`${patch_file}`"
            ${patch_file} --summary > ${BASEDIR}${PATCH_RECORD_DIR}/${PATCH}
            install_log "Finished installing patch ${PATCH}"
        fi
    done
}

#
# Function: start_product()
# Description: Manually start the product
# Args: none
#
start_product()
{
    install_log "start_product called"
    console_log "Stopping ${PRODUCT_NAME}"

    if [ -x ${PRODUCT_DIR}/init.d/${PRODUCT_BINARY}rc ]; then
    printf "Would you like to start ${PRODUCT_NAME} now (y/N)? "
    xor n y n

    if [ $? -eq 0 ]; then
        printf "Starting ${PRODUCT_NAME}: "
        sh ${PRODUCT_RC} stop
        sh ${PRODUCT_RC} start
    fi
    echo
    fi

    install_log "start_product returning"
}

#
# Function: rename_reboot()
# Description: Moves the reboot command out of the way and replaces with a script that
#              provides a simple warning
# Not used as yet.
# Args:
#       1) -i for install, -r for remove (put back as was)
#
rename_reboot()
{
    install_log "rename_reboot called"
    console_log "Disabling reboot command"

    INSTALL_REP="Y"
    REBOOT_CMD=""
    HALT_CMD=""
    MV_REBOOT_CMD=""
    MV_HALT_CMD=""
    BINTYPE=ELF
    export INSTALL_REP REBOOT_CMD HALT_CMD MV_REBOOT_CMD MV_HALT_CMD

    while [ $# -gt 0 ]; do
    case "$1" in
        -i) INSTALL_REP="Y"
        ;;

        -r) INSTALL_REP="N"
        ;;

        -*) echo "rename_reboot called with unknown option...${1}"
        ;;
    esac
    shift
    done

    #
    # Setup file names
    case "${COMPANY_OS}" in
    'sunos')
            REBOOT_CMD=/usr/sbin/reboot
            HALT_CMD=/usr/sbin/halt
            MV_REBOOT_CMD=/usr/sbin/rsforig_reboot
            MV_HALT_CMD=/usr/sbin/rsforig_halt
            BINTYPE=ELF
        ;;
    esac

    install_log "rename_reboot called"
}

#
# Function      : config_db_backup()
# Description   : Backup the database to a backup file.
# Args          : none
#
config_db_backup()
{
    console_log "Backing up ${CDB_PATH} to ${CDB_BACKUP}"
    if [ ! -f "${BASEDIR}${CDB_PATH}" ]; then
        console_log "No database at ${BASEDIR}${CDB_PATH} - skipping backup"
    else
        cp -f ${BASEDIR}${CDB_PATH} ${BASEDIR}${CDB_BACKUP}
    fi
}

#
# Function      : config_db_restore()
# Description   : Restore the database from a backup file.
# Args          : none
#
config_db_restore()
{
    console_log "Restoring ${CDB_PATH} from ${CDB_BACKUP}"
    if [ ! -f "${BASEDIR}${CDB_BACKUP}" ]; then
        console_log "No database backup at ${BASEDIR}${CDB_BACKUP} - skipping restore"
    else
        mv ${BASEDIR}${CDB_BACKUP} ${BASEDIR}${CDB_PATH}
    fi
}

#
# Function      : config_db_fixup()
# Description   : Generic function to fix issues in the config db
# Args          : none
#
config_db_fixup()
{
    console_log "Applying fixups to config DB"

    VAL_PROP_PGR3_DRIVE_COUNT=`${BASEDIR}${CDB_EXEC} -t ${BASEDIR}${CDB_PATH} -r 'pkg->installation' read prop_pgr3_drive_count`
    if [ "${VAL_PROP_PGR3_DRIVE_COUNT}" = "0" ]; then
        config_db_set prop_pgr3_drive_count update "-1"
    fi
}

#
# Function      : db2flat()
# Description   : Export an existing database to a flat-file using db2flat.
# Args          : none
#
db2flat()
{
    FLAT_PY="/opt/HAC/bin/flatdb.py"

    if [ -f "${BASEDIR}${FLAT_PY}" ]; then

        if [ -f "${BASEDIR}${CDB_PATH}" ] && [ ! -f "${BASEDIR}${CDB_FLAT}" ]; then

            console_log "Exporting ${CDB_PATH} to flat-file ${CDB_FLAT}"

            db2flat_out="db2flat : `${BASEDIR}${HAC_PYTHON} ${FLAT_PY} ${BASEDIR}${CDB_PATH} ${BASEDIR}${CDB_FLAT}`"
            [ $? -ne 0 ] && echo "${db2flat_out}"
            install_log "${db2flat_out}"
        fi
    else
        console_log "${FLAT_PY} not found - skipping import"
    fi
}

#
# Function      : remove_flat_file()
# Description   : Remove any existing flat-file database.
# Args          : none
#
remove_flat_file()
{
    if [ -f "${BASEDIR}${CDB_FLAT}" ]; then
        console_log "Removing flat file ${CDB_FLAT}"
        rm -f ${BASEDIR}${CDB_FLAT}
    fi
}

#
# Function      : config_db_export()
# Description   : Exports the config database to a flat file.
# Args          : 1) "clean" to remove original DB (and backup) after a
#                    successful export.
# Return        : 0 on success, 1 on error
#
config_db_export()
{
    option_str=$1

    CDB_INSTALL_FLAGS="-t ${BASEDIR}${CDB_PATH}"

    # Remove any existing flat-file to prevent any file conflict errors.
    #
    remove_flat_file

    ${BASEDIR}${CDB_EXEC} ${CDB_INSTALL_FLAGS} export_flat ${BASEDIR}${CDB_FLAT}

    # If the export was successful, remove the orignal database file and it's
    # backup file if 'clean' was provided to the option string.
    #
    if [ $? -eq 0 ]; then
        case $option_str in
            clean)
                console_log "Removing original database file/backup"
                rm -f ${BASEDIR}${CDB_PATH}
                rm -f ${BASEDIR}${CDB_BACKUP}
            ;;
        esac
    else
        console_log "DB EXPORT FAILED - leaving original database file(s)"
        return 1
    fi
    return 0
}

#
# Function      : config_db_import()
# Description   : Imports the config database if it is a flat file.
# Args          : none
# Return        : 0 on success, 1 on error
#
config_db_import()
{
    if [ "${CDB_TYPE}" = "gdbm" ]; then
        DB_UG_DIR=${BASEDIR}/${PRODUCT_DIR}/log/.db_upgrade
        install_log "Creating directory ${DB_UG_DIR}"
        rm -rf ${DB_UG_DIR}
        mkdir -p ${DB_UG_DIR}
        # Check a flat-file exists before continuing
        #
        if [ ! -f "${CDB_FLAT}" ]; then
            console_log "No flat-file found - skipping database import"
            return 1
        fi

        # If a database is found, back it up before continuing.
        if [ -f "${CDB_PATH}" ]; then
            config_db_backup

            if [ ! -f "${CDB_BACKUP}" ]; then
                console_log "No backup databse found - aborting import"
                return 1
            fi
        fi

        # Remove any existing database file to ensure a fresh import is
        # performed.
        #
        if [ -f "${BASEDIR}${CDB_PATH}" ]; then
            mv ${BASEDIR}${CDB_PATH} ${DB_UG_DIR}/db_before_convert
        fi
        cp ${BASEDIR}${CDB_FLAT} ${DB_UG_DIR}/db_flat

        # Import the flat-file database.
        #
        CDB_INSTALL_FLAGS="-d ${CDB_TYPE} -t ${BASEDIR}${CDB_PATH}"

        console_log "Importing flat-file ${CDB_FLAT}"

        ${BASEDIR}${CDB_EXEC} ${CDB_INSTALL_FLAGS} import_flat ${BASEDIR}${CDB_FLAT}

        # If the import was successful, remove the flat-file. If the import
        # failed, attempt to restore the original database from any available
        # backup.
        #
        if [ $? -eq 0 ]; then
            remove_flat_file
            cp ${BASEDIR}${CDB_PATH} ${DB_UG_DIR}/db_after_convert
        else
            config_db_restore
            return 1
        fi
    fi
    return 0
}

config_db_set()
{
    prop=$1
    action=$2
    if [ "$action" = "desc" ]; then
        description=$3
    else
        value=$3
        description=$4
    fi

    CDB_LOGSTR="pkg->installation"
    CDB_INSTALL_FLAGS="-t ${BASEDIR}${CDB_PATH} -r ${CDB_LOGSTR}"           # Round up of cdb flags.
    c_log_base="Creating database with default properties"
    u_log_base="Updating database properties"
    d_log_base="Deleting database properties"
    desc_log_base="Adding description to properties"

    if [ -z "${CDB_PATH}" -o -z "${CDB_CFG}" ]; then
        CDB_PATH="/opt/HAC/RSF-1/etc/.cluster.db"           # Path to config database itself.
        CDB_CFG="/opt/HAC/RSF-1/etc/.cluster_db.cfg"        # Path to config database configuration.
    fi

    case $action in
        create)
            console_log "${c_log_base} (${prop}: ${value})"
            ${BASEDIR}${CDB_EXEC} ${CDB_INSTALL_FLAGS} -s create ${prop} "${value}"
            if [ -n "${description}" ]; then
                ${BASEDIR}${CDB_EXEC} ${CDB_INSTALL_FLAGS} desc ${prop} "${description}"
            fi
            ;;
        update)
            console_log "${u_log_base} (${prop}: ${value})"
            ${BASEDIR}${CDB_EXEC} ${CDB_INSTALL_FLAGS} -s update ${prop} "${value}"
            ;;
        delete)
            console_log "${d_log_base} (${prop})"
            ${BASEDIR}${CDB_EXEC} ${CDB_INSTALL_FLAGS} -s delete ${prop}
            ;;
        desc)
            console_log "${desc_log_base} (${prop})"
            ${BASEDIR}${CDB_EXEC} ${CDB_INSTALL_FLAGS} desc ${prop} "${description}"
            ;;
    esac
}

#
# Function: config_db_create_cfg()
#
# Description:
# Save a copy of the type and location so rsfcdb on it's own will work.
# Note this is done WITHOUT basedir added to CDB_PATH so we save the
# correct path for an installed instance - we then use the BASEDIR
# prefix for creating properties so they are written into
# the relocatable copy of the database.
#
# Args: none
#
config_db_create_cfg()
{
    if [ -x "${BASEDIR}${CDB_EXEC}" ] ; then
	[ -z "${CDB_PATH}" ] && CDB_PATH="/opt/HAC/RSF-1/etc/.cluster.db"    ; export CDB_PATH
	[ -z "${CDB_CFG}"  ] && CDB_CFG="/opt/HAC/RSF-1/etc/.cluster_db.cfg" ; export CDB_CFG
        ${BASEDIR}${CDB_EXEC} -t ${CDB_PATH} -c ${BASEDIR}${CDB_CFG} -W
    fi
}

#
# Function: add_smf_services()
# Description: add any services we need to SMF - Solaris only.
# Args: none
#
add_smf_services()
{
    if [ -f ${SVCADM} ] ; then
        install_log " add_smf_services: adding stmfha services to SMF"

        # If the run-once SMF service doesn't exist, this must be the SVR-4 package
        # That means the manifest-import service won't be automatically restarted, so
        # we need to restart it now to import our various manifests
        if ! svcs run-once >/dev/null 2>&1; then
            console_log "Restarting manifest-import service"
            printf "%s" "Waiting for manifest-import service to restart." >> ${INSTALL_LOG}
            ${SVCADM} restart manifest-import
            sleep 1
            while ! svcs -H manifest-import | egrep '^online' >/dev/null; do
            printf "%s" "." >> ${INSTALL_LOG}
                sleep 1
            done
        echo done >> ${INSTALL_LOG}
        fi

        console_log "Enabling SMF services"
	if is_nef; then
            install_log " add_smf_services: svcadm enable stmfha, pyrpc, rsfsyseventd and nefliaison"
	else
            install_log " add_smf_services: svcadm enable stmfha, rsfsyseventd and rsf-rest"
	fi

        ${SVCADM} enable stmfha

	if is_nef; then
            ${SVCADM} enable RSF-1/pyrpc
	fi

        ${SVCADM} enable RSF-1/rsfsyseventd

	if is_nef; then
            ${SVCADM} enable RSF-1/nefliaison
	else
            ${SVCADM} enable RSF-1/rsf-rest
	fi
    fi
}

add_systemd_services()
{
    case "${COMPANY_OS}" in
    'linux')
        if which systemctl >/dev/null 2>&1; then
            if [ -f ${PRODUCT_DIR}/lib/rsf-rest.service ]; then
                cp ${PRODUCT_DIR}/lib/rsf-rest.service ${BASEDIR}/lib/systemd/system/rsf-rest.service
                systemctl daemon-reload >/dev/null 2>&1
                systemctl enable rsf-rest >/dev/null 2>&1
                systemctl start rsf-rest >/dev/null 2>&1
            fi
            if [ -f ${PRODUCT_DIR}/lib/rsf-1.service ]; then
                cp ${PRODUCT_DIR}/lib/rsf-1.service ${BASEDIR}/lib/systemd/system/rsf-1.service
                systemctl daemon-reload >/dev/null 2>&1
                systemctl enable rsf-1 >/dev/null 2>&1
                systemctl start rsf-1 >/dev/null 2>&1
            fi
            if which netplan >/dev/null 2>&1; then
                if [ -f ${PRODUCT_DIR}/lib/rsf-netplan.service ]; then
                    cp ${PRODUCT_DIR}/lib/rsf-netplan.service ${BASEDIR}/lib/systemd/system/rsf-netplan.service
                    systemctl daemon-reload >/dev/null 2>&1
                    # Do not enable it. Done via REST based on DB prop
                fi
            fi
        fi
        ;;

    *)
        install_log " Adding systemd services for the OS ${COMPANY_OS}"
        install_log " is not supported and therefore nothing has been done"
        ;;
    esac
}

remove_systemd_services()
{
    case "${COMPANY_OS}" in
    'linux')
        if which systemctl >/dev/null 2>&1; then
            if [ -f ${PRODUCT_DIR}/lib/rsf-rest.service ]; then
                systemctl stop rsf-rest >/dev/null 2>&1
                systemctl disable rsf-rest >/dev/null 2>&1
                rm -f /lib/systemd/system/rsf-rest.service
                systemctl daemon-reload >/dev/null 2>&1
            fi
            if [ -f ${PRODUCT_DIR}/lib/rsf-1.service ]; then
                systemctl stop rsf-1 >/dev/null 2>&1
                systemctl disable rsf-1 >/dev/null 2>&1
                rm -f /lib/systemd/system/rsf-1.service
                systemctl daemon-reload >/dev/null 2>&1
            fi
        fi
        ;;

    *)
        install_log " Removing systemd services for the OS ${COMPANY_OS}"
        install_log " is not supported and therefore nothing has been done"
        ;;
    esac
}

#
# Function: remove_smf_services()
# Description: add any services we need to SMF - Solaris only.
# Args: none
#
remove_smf_services()
{
    if [ -f ${SVCADM} ] ; then
	if is_nef; then
            install_log " remove_smf_services: Stopping neflisison, rsfsyseventd, pyrpc and stmfha services"
            SMF_SERVICES="RSF-1/nefliaison RSF-1/rsfsyseventd RSF-1/pyrpc stmfha"
	else
            install_log " remove_smf_services: Stopping rsfsyseventd, stmfha and rsf-rest services"
            SMF_SERVICES="RSF-1/rsfsyseventd stmfha RSF-1/rsf-rest"
	fi

        for svc in ${SMF_SERVICES}; do
            install_log "${SVCADM} disable -s ${svc}"
            console_log "Disabling SMF services (${svc})"
            ${SVCADM} disable -s ${svc}
        done

        rm -f ${BASEDIR}lib/svc/method/svc-rsfsyseventd \
              ${BASEDIR}lib/svc/method/svc-stmfha \
              ${BASEDIR}lib/svc/method/svc-pyrpc \
              ${BASEDIR}lib/svc/method/svc-rsf-rest \
              ${BASEDIR}lib/svc/method/svc-nefliaison \
              ${BASEDIR}var/svc/manifest/application/rsf-1.xml \
              ${BASEDIR}var/svc/manifest/application/rsfsyseventd.xml \
              ${BASEDIR}var/svc/manifest/application/stmfha.xml \
              ${BASEDIR}var/svc/manifest/application/pyrpc.xml \
              ${BASEDIR}var/svc/manifest/application/rsf-rest.xml \
              ${BASEDIR}var/svc/manifest/application/nefliaison.xml
        install_log " remove_smf_services: restarting manifest-import to remove services from SMF"
        console_log "Restarting manifest-import service"
        ${SVCADM} restart manifest-import
    fi
}

#!/bin/sh

#
# Function    : check_svc_online()
# Description : Checks if a Solaris SMF service is in a steady online state.
#               Waits for a defined time period for the service to become steady.
# Args        :
#               $1  Name of the SMF service to be checked.
#               $2  Timeout limit in seconds.
# Returns     :
#               0   Service is online.
#               1   Service is not online.
#               2   Service is not online but is in transition and may be online soon.
#               3   Insufficient arguments provided.
#
check_svc_online()
{
    # Check that enough arguments were provided
    if [ "$#" -lt 2 ] ; then
        echo "Insufficient args args passed to function"
        return 3
    fi

    # Set the SMF service name and timeout value
    service=$1
    timeout=$2

    # Initialise the internal counter and transit status
    counter=0
    transit=0

    # Loop until the timeout has expired
    while [ $counter -lt ${timeout} ]
    do
        # Check the state of the SMF service has started
        status=$"( svcs -H -o state ${service} )"

        echo "attempt `expr ${counter} + 1` of ${timeout} [ Current state : ${status} ]"

        # If the service is in a transition state, set the transition status and
        # wait for a second before checking the status again
        case "${status}" in
        *\**)
            transit=1
            sleep 1
            ;;
        *)
            # Not in a transitional state - clear the transit status and check
            # if the service is online. If the service is online, return an OK
            # response. Otherwise, continue to process the loop.

            transit=0

            case "${status}" in
            'online')
                echo "  $service SMF service started successfully"
                return 0
                ;;
            *)
                sleep 1
                ;;
            esac
        ;;
    esac

    counter=`expr ${counter} + 1`
    done

    # A timeout has occurred. If the last attempt reported the service was in
    # a transit state, return 2 to indicate the service didn't go online steady
    # but may do so soon. Otherwise, return 1 to indicate the service was steady
    # but not in an online state.
    if [ transit -eq 0 ]; then
        echo "  Failed to start the service"
        return 1
    else
        echo "  Timed out waiting for ${service} to report a steady online state"
        return 2
    fi
}

#
# Function    : install_python_requests_module()
# Description : Installs the requests python module required by the python API
#               to perform RESTful calls.
# Args        : none
#
install_python_requests_module()
{
    PYTHON_LIB_DIR=/opt/HAC/RSF-1/lib/python
    REQUESTS_TAR=requests.tar
    console_log "Installing Python requests module"

    if [ -f ${PYTHON_LIB_DIR}/${REQUESTS_TAR} ] ; then
        install_log " install_python_requests_module: installing requests module"
        (cd ${PYTHON_LIB_DIR}; tar xf ${REQUESTS_TAR})
        rm -f ${PYTHON_LIB_DIR}/${REQUESTS_TAR}
    fi
}

#
# Function    : remove_python_requests_module()
# Description : Removes the requests python module required by the python API
#               to perform RESTful calls.
# Args        : none
#
remove_python_requests_module()
{
    console_log "Removing Python requests module"
    rm -rf /opt/HAC/RSF-1/lib/python/requests
}

#
# Function    : install_http_server()
# Description : Installs the RSF-1 GUI server and any files it needs to serve.
# Args        : none
#
install_http_server()
{
    if [ -f ${BASEDIR}opt/HAC/RSF-1/SVC/rsf_standalone.tar ]; then
        console_log "Enabling Python RPC server"

        case ${COMPANY_OS} in
            sunos)
                ${SVCADM} enable RSF-1/pyrpc
                if grep Nexenta /etc/release >/dev/null; then
                    return
                fi
                ;;
        esac

        console_log "Generating config file for HTTP server"

        INSTALL_DIR=/opt/HAC/RSF-1/SVC
        OUTFILE=${BASEDIR}${INSTALL_DIR}/rsf_standalone.conf
        SOCK_PORT=8020
        SOCK_ADDR=0.0.0.0

        STATICDIR=${INSTALL_DIR}/static/
        HTML_INDEX=standalone/index.html
        FAVICON_DIR=html/favicon.ico

        # Extract standalone server files
        #
        (cd ${BASEDIR}${INSTALL_DIR}; tar xf rsf_standalone.tar)

        # Generate CherryPy config file
        #
        echo "[global]" > ${OUTFILE}
        echo "server.socket_port: ${SOCK_PORT}" >> ${OUTFILE}
        echo "server.socket_host: '${SOCK_ADDR}'" >> ${OUTFILE}
        echo "" >> ${OUTFILE}

        echo "[/]" >> ${OUTFILE}
        echo "tools.staticdir.on: True" >> ${OUTFILE}
        echo "tools.staticdir.dir: '${STATICDIR}'" >> ${OUTFILE}
        echo "tools.staticfile.root: '${STATICDIR}'" >> ${OUTFILE}
        echo "tools.staticdir.index: '${HTML_INDEX}'" >> ${OUTFILE}
        echo "" >> ${OUTFILE}

        echo "[/favicon.ico]" >> ${OUTFILE}
        echo "tools.staticfile.on: True" >> ${OUTFILE}
        echo "tools.staticfile.filename: '${FAVICON_DIR}'" >> ${OUTFILE}

        # Extract the extJS files
        #
        if [ -f ${BASEDIR}${STATICDIR}extern/extjs-3.tar ] ; then
            install_log " install_http_server: extracting extJS"
            (cd ${BASEDIR}${STATICDIR}extern; tar xf extjs-3.tar)
        fi

        # Generate the HTML files
        #
        HTML_DIR="${BASEDIR}${STATICDIR}html"
        HTML_GEN=html-gen.sh

        if [ -f ${HTML_DIR}/${HTML_GEN} ] ; then
            . ${HTML_DIR}/${HTML_GEN}

            HTML_PAGES="Advanced Diagnostics Heartbeats Initialise Status Volumes"
            HTML_ENV="standalone"

            #if is_nexenta ; then
            if [ -d /var/lib/nza ] ; then
                console_log "Generating HTML for HTTP server (nexenta)"
                HTML_ENV="nexenta"
            else
                console_log "Generating HTML for HTTP server (standalone)"
            fi

            for PAGE in ${HTML_PAGES} ; do
                (cd ${HTML_DIR};generate_html_page ${PAGE} ${HTML_ENV})
            done
        fi

        # Attempt to start the RSF-1 GUI server now. Use a restart command to start
        # the server to avoid any errors being returned if the GUI was already
        # running
#        install_log " install_http_server: checking XML-RPC server is running..."


        # Make the management script executable
        #RSFGUI_RC=${BASEDIR}${INSTALL_DIR}/init.d/rsfsg_rc.pyc
        #chmod 755 ${RSFGUI_RC}

        case ${COMPANY_OS} in
        sunos)
            # Install the SMF management script
            install_log " install_http_server: installing SMF script for the GUI"
            cp ${BASEDIR}opt/HAC/RSF-1/lib/xml/rsf-gui ${BASEDIR}lib/svc/method/rsf-gui
            chmod 755 ${BASEDIR}lib/svc/method/rsf-gui

            console_log "Importing rsf-gui SMF service"
            svccfg import ${BASEDIR}opt/HAC/RSF-1/lib/xml/rsf-gui.xml
            ;;
        esac
    fi
}

install_webapp()
{
    INSTALL_DIR=/opt/HAC/RSF-1/gui
    TAR_FILE=webapp-build.tar.gz

    if [ -f ${BASEDIR}${INSTALL_DIR}/${TAR_FILE} ]; then
        console_log "Installing web app"

        OUTFILE=${BASEDIR}${INSTALL_DIR}/bs-config.json

        # Extract standalone server files
        #
        (cd ${BASEDIR}${INSTALL_DIR}; tar xfz ${TAR_FILE})

        # Rewrite server config
        #
        echo "{" > ${OUTFILE}
        echo "  \"port\": 3000," >> ${OUTFILE}
        echo "  \"files\": [\"./**/*.{html,htm,css,js}\"]," >> ${OUTFILE}
        echo "  \"server\": {" >> ${OUTFILE}
        echo "    \"baseDir\": \".\"," >> ${OUTFILE}
        echo "    \"routes\": {" >> ${OUTFILE}
        echo "        \"/node_modules\": \"node_modules\"," >> ${OUTFILE}
        echo "        \"/sb-admin\" : \"sb-admin-2\"" >> ${OUTFILE}
        echo "    }" >> ${OUTFILE}
        echo "  }," >> ${OUTFILE}
        echo "  \"https\": {" >> ${OUTFILE}
        echo "    \"key\" : \"host.key\"," >> ${OUTFILE}
        echo "    \"cert\" : \"host.cert\"" >> ${OUTFILE}
        echo "  }" >> ${OUTFILE}
        echo "}" >> ${OUTFILE}
    fi
}

remove_webapp()
{
    console_log "Removing web app"
    rm -rf ${BASEDIR}/opt/HAC/RSF-1/gui
}

#
# Function    : start_restapi()
# Description : Attempts to start the RSF-1 Rest API and Web Application process.
# Args        : none
#
start_restapi()
{
    case "${COMPANY_OS}" in
    'sunos')
        console_log "Starting RSF-1 Rest API and Web Application"
        ${SVCADM} disable rsf-rest >/dev/null 2>&1
        ${SVCADM} enable -s rsf-rest > /tmp/rsf-rest-start-err 2>&1
        if [ "$?" -eq 0 ] ; then
            console_log "RSF-1 Web Application now available on http://`uname -n`:4330/"
            echo
        else
            console_log "Failed to start Rest API and Web Application server."
            cat /tmp/rsf-rest-start-err >> ${INSTALL_LOG}
            echo
            cat /tmp/rsf-rest-start-err
            echo
        fi
        ;;
    esac
}


#
# Function    : remove_http_server()
# Description : Removes the RSF-1 GUI server and it's served content.
# Args        : none
#
remove_http_server()
{
    console_log "Removing HTTP server"
    case "${COMPANY_OS}" in
    'sunos')
        SVC_DIR=${BASEDIR}opt/HAC/RSF-1/SVC

        /opt/HAC/RSF-1/bin/python ${BASEDIR}${SVC_DIR}/init.d/rsfsg_rc.pyc stop >/dev/null 2>&1

        rm -rf ${BASEDIR}opt/HAC/RSF-1/SVC

        install_log " remove_http_server: removing rsf-gui services from SMF"

        ${SVCADM} disable rsf-gui >/dev/null 2>&1
        ${SVCADM} clear   rsf-gui >/dev/null 2>&1
        ${SVCADM} disable rsf-gui >/dev/null 2>&1
        svccfg delete  rsf-gui >/dev/null 2>&1

        rm -f ${BASEDIR}lib/svc/method/rsf-gui ${BASEDIR}var/svc/manifest/network/rsf-gui.xml
        ;;
    esac
}

#
# Function    : install_hac_python
# Description : Install Python interpreter to /opt/HAC/Python
# Args        : none
#
install_hac_python()
{
    console_log "Installing Python interpreter to /opt/HAC/Python"

    if ! grep -i 'nexenta.*[^.]5' /etc/release >/dev/null 2>&1 ; then
        OWD=${PWD}
        cd ${BASEDIR}/opt/HAC
        tar xzf ${BASEDIR}/opt/HAC/RSF-1/lib/hac_python.tar.gz
    if [ -d "${BASEDIR}/opt/HAC/Python" ]; then
        rm -rf Python
    fi
        mv install/ Python
        cd ${OWD}
    fi

    if [ -f /opt/HAC/Python/bin/python3 ]; then
        install_log " install_hac_python: /opt/HAC/RSF-1/bin/python --> /opt/HAC/Python/bin/python3"
        ln -sf /opt/HAC/Python/bin/python3 /opt/HAC/RSF-1/bin/python
    elif [ -f /usr/bin/python3.4 ]; then
        install_log " install_hac_python: /opt/HAC/RSF-1/bin/python --> /usr/bin/python3.4"
        ln -sf /usr/bin/python3.4 /opt/HAC/RSF-1/bin/python
    else
        install_log " install_hac_python: /opt/HAC/RSF-1/bin/python --> /usr/bin/python"
        ln -sf /usr/bin/python /opt/HAC/RSF-1/bin/python
    fi
}

remove_hac_python()
{
    console_log "Removing Python interpreter from /opt/HAC/Python"

    rm -f /opt/HAC/RSF-1/bin/python
    rm -rf /opt/HAC/Python
}

#
# Function: cleanup_core_files()
# Description: Core files from an older package are useless so we can delete them.
#              It may be useful to keep a log of exactly what we are deleting though
# Args: none
cleanup_core_files()
{
    CORE_DIR=/var/cores
    if [ -d ${CORE_DIR} ]; then
        core_types="`ls ${CORE_DIR}|nawk '/core\.rsf/\
        {
            FS=\".\"
            if (\" \" types \" \" !~ \" \" $2 \" \")
                types = types \" \" $2
        }
        END {
            print types
        }'`"
        for type in ${core_types}; do
            num_cores=`ls ${CORE_DIR}/core.${type}*|wc -l|awk '{print $1}'`
            console_log "Removing ${num_cores} ${type} core files from ${CORE_DIR}"
            rm -f ${CORE_DIR}/core.${type}.*
        done
    fi
}

stop_fen()
{
    if [ -x ${SVCADM} ]; then
        fen_state="`svcs -Ho state fen 2>/dev/null`"
        if [ $? -eq 0 ]; then
            if [ "$fen_state" = "online" ]; then
                console_log "Disabling FEN service"
                ${SVCADM} disable -s fen
            else
                install_log "FEN service not enabled (state = $fen_state)"
            fi
        else
            console_log "FEN service not installed"
        fi
    else
        install_log "FEN service not provided for this platform"
    fi
}

#############################################################################
#
# Function:    check_dtrace()
#
# Description: Function to test dtrace to make sure its output contains
#              the exit code of child process.
#              Contents of the output file (-o) look like:
#  CPU     ID                    FUNCTION:NAME
#    3      1                           :BEGIN 2019 Nov  4 12:38:28
#
#    3      2                             :END 2019 Nov  4 12:38:28
#
# zpool_exit_code 1
#
# Args:	       none
#
# Return:      0 - DTrace is OK to use
#              1 - DTrace is broken or doesn't exist on this system
#
check_dtrace()
{
	DT_SCRIPT=${RSF_DTRACE_DIR}/spa_import.d
	if ! [ -f ${DT_SCRIPT} ]; then
		DT_SCRIPT=${DTRACE_DIR}/spa_import.d
	fi

	if ! [ -f ${DT_SCRIPT} ]; then
		dated_echo "spa_import.d does not exist"
		return 1
	elif ! which dtrace >/dev/null 2>&1; then
		dated_echo "dtrace does not exist"
		return 1
	fi

	DTLOG=/tmp/dtrace_test.$$.log

	dtrace -s ${DT_SCRIPT} -o ${DTLOG} -Z -c "false" >/dev/null 2>&1
	cmd_exit_code=`awk "${DTRACE_CODE_EXTRACTOR}" ${DTLOG}`
	rm -f ${DTLOG}

	if [ ${cmd_exit_code} -gt 0 ]; then
		return 0
	else
		dated_echo "Failed to get exit code from dtrace"
		return 1
	fi
}

#
# Function: do_rsf_postinstall()
# Description: Postinstall for RSF. Note that this is here instead of just in the
#              postinstall script as linux does not call the post install script from
#              RFC's, so we have to call it directly.
# Args: none
#
do_rsf_postinstall()
{

    install_log "do_rsf_postinstall: called"

    if [ -f "${BASEDIR}opt/HAC/.version" ] ; then
        INSTALL_VERSION=`cat ${BASEDIR}opt/HAC/.version`
        install_log "do_rsf_postinstall: this install log is for version ${INSTALL_VERSION}"
    fi

    cleanup_core_files

    install_hac_python

    if is_nef; then
        #
        # For NEF, we no longer use FEN, so we can remove the FEN log files
        #
	stop_fen
        console_log "Removing FEN log files"
        rm -f ${BASEDIR}opt/HAC/RSF-1/log/fen.log*
    fi

    boot_link -e

    add_rsf_services

    if [ -f ${BASEDIR}opt/HAC/RSF-1/etc/.disable_disc_heartbeats ] ; then
        install_log " do_rsf_postinstall: rm ${BASEDIR}opt/HAC/RSF-1/etc/.disable_disc_heartbeats"
        rm -f ${BASEDIR}opt/HAC/RSF-1/etc/.disable_disc_heartbeats >/dev/null 2>&1
    fi

    create_rsf_account _rsfadmin 'WF4wPrvthykjglbeM2i.4/djbtYBua5L'

    create_rsf_default_config

    #
    # Create a configuration file for the DB.
    config_db_create_cfg

    if is_nef; then
	#
	# Seed the DB (fresh install)
	/opt/HAC/RSF-1/bin/python ${BASEDIR}opt/HAC/RSF-1/lib/python/update_service_environment_db.pyc

	#
	# Fixes the issue where we have PGR3 set to 0 (rather than -1) for all drives.
	config_db_fixup
    else
    	config_db_defaults
    fi

    link_mapmgr

    install_patches

    case "${COMPANY_OS}" in
    'linux')
        add_systemd_services
        ;;
    *)
        add_smf_services
        ;;
    esac

    #
    # 04/11/2019
    # Check to ensure dtrace exists and the mechanisum we use to grab
    # the exit code from commands run also works.  We do this once on
    # every install,  and update the  PROP_DTRACE_IS_BROKEN  property
    # appropriately.  Finally, in rsf.sh we check if dtrace is broken
    # and  if  it  is  we turn off using a dtrace zpool import/export
    # wrapper (PROP_ZPOOL_DTRACE_IMPORT_EXPORT).
    if is_nef; then
	if ! check_dtrace ; then
	    ${CDB_EXEC} ${CDB_FLAGS} update prop_dtrace_is_broken ${BOOL_STR_TRUE}
	else
	    ${CDB_EXEC} ${CDB_FLAGS} update prop_dtrace_is_broken ${BOOL_STR_FALSE}
	fi
    fi

    if ! is_nef; then
	install_webapp
    fi

    restart_rsf

    if ! is_nef; then
	start_restapi
    fi

    install_log "do_rsf_postinstall: returning"
    echo
    return 0
}

#
# Function: do_rsf_preremove()
# Description: Preremove for RSF. Note that this is here instead of just in the
#              preremove script as linux does not call the pre remove script from
#              RFC's, so we have to call it directly.
# Args: none
#
do_rsf_preremove()
{
    install_log "do_rsf_preremove called"

    #
    # Run the product stop script in a seperate shell so exit 0
    # does not exit this script!
    stop_rsf

    unlink_mapmgr

    case "${COMPANY_OS}" in
    'linux')
        remove_systemd_services
        ;;
    *)
        remove_smf_services
        ;;
    esac

    if ! is_nef; then
	remove_webapp
    fi

    boot_link -d

    if is_nef; then
	config_db_export "clean"
    fi

    remove_hac_python

    install_log "do_rsf_preremove returning"
    echo

    return 0
}
fi
