#!/bin/sh
#
# $Id$
#
# Script:	rsf.sh
#
# Description:	Generic supporting vars and functions for RSF-1 scripts
#
# Platform:	Unix
#
# Author:	High-Availability Ltd
#

#############################################################################
# Variable definitions
#############################################################################
#
# Set lang to C so we understand log messages etc.
LANG="C"
export LANG

#
# Path settings.
COMPANY_DIR="/opt/HAC"                                    # Base install directory
RSF1_HOME="/opt/HAC"                                      # Base install directory - to replace COMPANY_DIR
COMPANY_BIN="${COMPANY_DIR}/bin"                          # Base binary dirctory
PRODUCT_DIR="${COMPANY_DIR}/RSF-1"	                  # RSF-1 base installation directory
PRODUCT_BIN="${COMPANY_DIR}/RSF-1/bin"                    # RSF-1 binary dirctory
PRODUCT_ETC="${COMPANY_DIR}/RSF-1/etc"                    # RSF-1 etc directory
PRODUCT_LIB="${COMPANY_DIR}/RSF-1/lib"                    # RSF-1 lib directory
PRODUCT_RUN="${COMPANY_DIR}/RSF-1/run"                    # RSF-1 run directory
PRODUCT_MOD="${COMPANY_DIR}/RSF-1/mod"                    # RSF-1 kernel modules
PRODUCT_LOGDIR="${COMPANY_DIR}/RSF-1/log"                 # RSF-1 log directory
PRODUCT_PYTHON_LIB="${PRODUCT_LIB}/python"                # RSF-1 python lib dir
STMFHA_LOGDIR="${PRODUCT_LOGDIR}/stmfha"                  # Map manager log directory (was MAPMGR_LOGDIR)
BROKEN_LOG="${PRODUCT_LOGDIR}/broken.log.d"               # Directory to keep reason for broken unsafe
DTRACE_LOGDIR="${PRODUCT_LOGDIR}/dtrace"                  # Directory to keep dtrace log files from slow import/export
METROHA_DIR="${PRODUCT_ETC}/.MetroHA"                     # Directory for MetroHA specific files
PRODUCT_INIT="${COMPANY_DIR}/RSF-1/init.d"                # RSF-1 /etc/init.d like directory
PRODUCT_CONFIG="${PRODUCT_ETC}/config"                    # Location of configuration file
PRODUCT_RELEASE="${PRODUCT_ETC}/.release"                 # Location of RSF-1 current release file
PRODUCT_RC="${PRODUCT_INIT}/rsfmonrc"                     # Location of RSF-1 startup script (linked to /etc/init.d)
MHDC_CACHE="${PRODUCT_ETC}/.MHDC.cache"                   # Cache directory for ZFS MHDC devices
RSFPMON="${PRODUCT_BIN}/rsfpmon"                          # Rsf process monitor binary
STMFPROXY="/usr/demo/comstar/bin/svc-stmfproxy"           # Proxy SVC start/stop script
STMFPROXY_CONFIG="${PRODUCT_ETC}/ALUA.cfg"                # Location of STMF proxy configuration file
STMFPROXY_CONFIG_LEGACY="${PRODUCT_ETC}/.stmf_proxy"      # Location of old STMF proxy cofig file, for backwards compatibility
[ ! -f ${STMFPROXY_CONFIG} -a -f ${STMFPROXY_CONFIG_LEGACY} ] && STMFPROXY_CONFIG=${STMFPROXY_CONFIG_LEGACY}
STMFPROXY_OFF="${PRODUCT_ETC}/disable.stmf_proxy"         # Location of STMF proxy configuration file when disabled
STMFPROXY_LOG="${STMFHA_LOGDIR}/stmf_proxymon.log"        # Location of the STMF proxy log file
STMFPROXY_DEBUG_ON="${PRODUCT_ETC}/.stmf_proxy_debug_on"  # Location of STMF proxy debug flag file - adds -d option to proxy
STMFPROXY_LOCKFILE="/var/run/rsfpmon"                     # rsfpmon lockfile location for stmf proxy
ALUAADM="/usr/demo/comstar/bin/aluaadm"                   # Location of ALUAADM binary

VCENTRE_CONFIG="${PRODUCT_ETC}/.vcnt"                     # Location of vCentre HA config
VHA="${PRODUCT_BIN}/vha"                                  # Location of the VHA binary

CACHEDIR="${PRODUCT_ETC}/volume-cache"                    # Cache directory for ZFS device cache
MHDC_RESERVATION_DRVS="${PRODUCT_ETC}/.res_drives"        # Specification of drives to use for reservations, .${service} added to end.
PATCH_DIR="${PRODUCT_DIR}/Patches"                        # Directory to place patches. Patches will be installed when a new RSF-1 package is installed.
PATCH_RECORD_DIR="${PATCH_DIR}/.installed"                # Directory to define which patches have already been installed.
CDB_EXEC="${PRODUCT_BIN}/rsfcdb"                          # Path to config database exec.
CDB_PATH="${PRODUCT_ETC}/.cluster.db"                     # Path to config database itself.
CDB_BACKUP="${PRODUCT_ETC}/.backup.cluster.db"            # Path to backup config database.
CDB_CFG="${PRODUCT_ETC}/.cluster_db.cfg"                  # Path to config database configuration.
CDB_FLAT="${PRODUCT_ETC}/.cluster.flat"                   # Path to use for the database flat-file.
PROG_MHDC="${PRODUCT_BIN}/mhdc"                           # MHDC command path
PROG_RSFCLI="${PRODUCT_BIN}/rsfcli"                       # RSF CLI path
PROG_RSFSYSEVENTD="${PRODUCT_BIN}/rsfsyseventd"           # Path to rsfsyseventd
PROG_RSFPYUTILS="${PRODUCT_BIN}/rsfpyutils.pyc"           # Path to RSF pyutils
DATE="${PRODUCT_BIN}/hac_date"                            # Print seconds since epoch (no portable way in shell)
HAC_PYTHON="/opt/HAC/RSF-1/bin/python"                    # Path to python interpreter instaled with RSF package
RPC_CLUSTER_STMF="${PRODUCT_BIN}/rpcstmfha"               # STMFHA server.
RPC_CLUSTER_STMF_LOG="${PRODUCT_LOGDIR}/rpcstmf.log"      # STMFHA server log file.
RPC_CLUSTER_STMF_LOCK="/var/run/rsfpmon_rpcstmfha"        # STMFHA server lock file
RPC_CLUSTER_SERVICES="${PRODUCT_BIN}/rpchasvc"            # HA RPC server.
RPC_CLUSTER_SERVICES_LOG="${PRODUCT_LOGDIR}/rpchasvc.log" # HA RPC server log file.
RPC_CLUSTER_SERVICES_LOCK="/var/run/rsfpmon_rpchasvc"     # HA RPC server lock file.
RPC_PYTHON_SERVICES="${PRODUCT_PYTHON_LIB}/rsf_server/RPCserver.pyc"       # HA Python RPC server.

RPC_PYTHON_CTL="${PRODUCT_BIN}/rsfrpc_rc.pyc"             # Start/Stop/Kill/Status for python server.

RPC_PYTHON_SERVICES_LOG="${PRODUCT_LOGDIR}/rpc_py.log"    # HA Python RPC server log file - depricated.
RPC_PYTHON_SERVICES_LOCK="/var/run/rsfpmon_rpc_py"        # HA Python RPC server lock file.

DONOTSTARTHA="/do-not-start-ha"                           # Do not start RSF-1 sentinel file.
PYTHON_LIBS="${PRODUCT_PYTHON_LIB}/rsf_cluster.zip"       # ZIP file with RSF Python modules

export BROKEN_LOG DONOTSTARTHA

#
# PATHS to external programmes and files not
# in PRODUCT_DIR
PRODUCT_PID="/var/run/rsfmon.pid"	                  # Location of saved PID
PRODUCT_DEBUG="/tmp/rsfmondebug"                          # Location of file to reset debugging (2.x)
SYS_EVENT_ADM="/usr/sbin/syseventadm"                     # Location of sys event handler (Solaris)

PRODUCT="RSF-1"                                           # Main product name
PRODUCT_NAME="RSF"		                          # Product script tag
PRODUCT_LOWCASE="rsf"                                     # Product name as lower case
PRODUCT_UPCASE="RSF"                                      # Product name as upper case
PRODUCT_PREFIX="rsf"		                          # Product filename prefix for binaries
PRODUCT_SCRIPT="rsf.sh"		                          # Product script definition file - this filename
PRODUCT_BINARY="rsfmon"		                          # Main product binary filename
RSF1_LOG="rsfmon.log"                                     # Name of RSF-1 log file
STMFHA_LOG="stmfha.log"                                   # Name of map manager log file
CTLD_LOG="ctld.log"                                       # Name of CTLD script log file
FEN_LOG="fen.log"                                         # Name of fen logs.
COMPANY="HAC"				                  # Company name - install dir & scripts
COMPANY_NAME="High-Availability.Com"	                  # Company name for printing stuff
COMPANY_DOMAIN="high-availability.com"	                  # Company Domain for email and web

CACHE_SUFFIX="cache"                                      # Suffix for the import cache file
CACHE_LIVE_SUFFIX="cache-live"                            # Suffix for the live (current) import cache file
CACHE_MD5_SUFFIX="md5"                                    # Suffix for the MD5 sum.
CACHE_SYNC_PIDFILE_PREFIX="rsf-cache-sync-pid"            # Cache sync pid file name (partial)
CDB_TYPE="gdbm"                                           # Default type of db we use.
CDB_FLAGS="-d ${CDB_TYPE} -t ${CDB_PATH}"                 # Round up of cdb flags.
BOOL_STR_TRUE="true"                                      # Standardise string flags - true
BOOL_STR_FALSE="false"                                    # Standardise string flags - false
BOOL_STR_NULL="null"                                      # Standardise string flags - null
ENFAILFAST_MICRO=100                                      # ENFAILFAST check time in milliseconds
SVCPROP="/usr/bin/svcprop"                                # SVC properties command - Solaris only
SVCADM="/usr/sbin/svcadm"                                 # SVC admin command - Solaris only
SVCS="/usr/bin/svcs"                                      # SVC status command - Solaris only

DTRACE_DIR=/usr/bin
RSF_DTRACE_DIR=${PRODUCT_LIB}

#
# Default values for properties. Do not need to be exported
# as package.sh is the only other place they are used and this
# is sourced.
PROP_ABORT_UNFENCED_IMPORT__VALUE="${BOOL_STR_TRUE}"
PROP_ADDITIONAL_COMMON_DIRECTORIES__VALUE="${BOOL_STR_NULL}"
PROP_CLUSTER_HOOK__VALUE="${BOOL_STR_NULL}"
PROP_COMSTAR_STANDBY_LUNS_BEFORE_FAILOVER__VALUE="${BOOL_STR_FALSE}"
PROP_COMSTAR_SUPPORT__VALUE="${BOOL_STR_TRUE}"
PROP_EVENT_NOTIFY__VALUE="${BOOL_STR_TRUE}"
PROP_FPING_RETRIES__VALUE="0"
PROP_GUI_CLUSTER_INITIALIZED__VALUE="${BOOL_STR_FALSE}"
PROP_GUI_DEFAULT_CLUSTER_NAME__VALUE="HA-Cluster"
PROP_GUI_DEFAULT_INITTIMEOUT__VALUE="20"
PROP_GUI_DEFAULT_RUNTIMEOUT__VALUE="8"
PROP_GUI_DIAGNOSTICS_TEST__VALUE="${BOOL_STR_FALSE}"
PROP_GUI_DISABLE_ONLINE_HELP__VALUE="${BOOL_STR_FALSE}"
PROP_GUI_ENABLE_COMSTAR_BROWSER__VALUE="${BOOL_STR_FALSE}"
PROP_GUI_ENABLE_DIAGNOSTICS__VALUE="${BOOL_STR_FALSE}"
PROP_GUI_ENABLE_FASTFAIL__VALUE="${BOOL_STR_FALSE}"
PROP_GUI_ENABLE_IPV6__VALUE="${BOOL_STR_FALSE}"
PROP_GUI_FAILOVER_STATE_STICKY__VALUE="${BOOL_STR_FALSE}"
PROP_GUI_JAVASCRIPT_DEBUG__VALUE="${BOOL_STR_FALSE}"
PROP_GUI_PLUGIN_VERSION__VALUE="HA PLUGIN VERSION 4.1.0, built `date +'%d %B %Y'`"
PROP_GUI_RES_DRIVE_SUPPORT__VALUE="${BOOL_STR_FALSE}"
PROP_GUI_SERIAL_HB_SUPPORT__VALUE="${BOOL_STR_FALSE}"
PROP_MHDC_STEC_SKIP__VALUE="${BOOL_STR_FALSE}"
PROP_MHDC_USE_FAILFAST__VALUE="${BOOL_STR_TRUE}"
PROP_MHDC_USE_PGR3__VALUE="${BOOL_STR_TRUE}"
PROP_PGR3_DRIVE_COUNT__VALUE="-1"
PROP_POOL_HOOK__VALUE="${BOOL_STR_NULL}"
PROP_PYTHON_RPC_SERVER_PORTNO__VALUE="4330"
PROP_RSF_ZFS_EVENT_LOGGING__VALUE="${BOOL_STR_FALSE}"
PROP_SCSI2_DRIVE_COUNT__VALUE="2"
PROP_SCSI2_RETRY_COUNT__VALUE="0"
PROP_RES_DRIVE_TEST_RETRIES__VALUE="3"
PROP_RES_DRIVE_TEST_DELAY__VALUE="100"
PROP_SCSI2_THREADED_RESERVE__VALUE="${BOOL_STR_FALSE}"
PROP_SCSI2_THREADED_RESERVE_COUNT__VALUE="-1"
PROP_SKIP_PGR3_LOG_AND_CACHE__VALUE="${BOOL_STR_TRUE}"
PROP_SKIP_VIP_CHECK__VALUE="${BOOL_STR_FALSE}"
PROP_SSH_BOUND__VALUE="${BOOL_STR_FALSE}"
PROP_SYNC_RSFCDB__VALUE="${BOOL_STR_FALSE}"
PROP_USE_ZFS_CACHE__VALUE="${BOOL_STR_TRUE}"
PROP_ZPOOL_DTRACE_IMPORT_EXPORT__VALUE="${BOOL_STR_FALSE}"
PROP_ZPOOL_DTRACE_NEEDED_TIMEOUT_SECONDS__VALUE="15"
PROP_ZPOOL_EXPORT_OPTION_C__VALUE="${BOOL_STR_FALSE}"
PROP_ZPOOL_FAIL_MODE__VALUE="panic"
PROP_ZPOOL_IMPORT_OPTION_M__VALUE="${BOOL_STR_FALSE}"
PROP_ZPOOL_THREADS_NUMBER__VALUE="100"
PROP_ZPOOL_THREADS_OPTION_T__VALUE="${BOOL_STR_FALSE}"
PROP_ZPOOL_SYNC_CACHE__VALUE="${BOOL_STR_TRUE}"
PROP_RES_DRIVE_WAIT__VALUE="0"
PROP_NFS_FAILOVER_SUPPORT__VALUE="${BOOL_STR_FALSE}"
PROP_DISABLE_SITE_SAFE__VALUE="${BOOL_STR_FALSE}"
PROP_DISABLE_DRIVE_MONITOR__VALUE="${BOOL_STR_FALSE}"
PROP_FORCE_IFCONFIG__VALUE="${BOOL_STR_FALSE}"
PROP_AUTO_UPDATE_MON_EXPRS__VALUE="${BOOL_STR_TRUE}"

#
# Exit codes/RSF-1 mappings
COMPANY_OK=0
COMPANY_WARN=1
COMPANY_RESTART=2
COMPANY_ABORT=3
COMPANY_SAFE=4
COMPANY_UNSAFE=5

RSF_OK=${COMPANY_OK}
RSF_WARN=${COMPANY_WARN}
RSF_RESTART=${COMPANY_RESTART}
RSF_ABORT=${COMPANY_ABORT}
RSF_SAFE=${COMPANY_SAFE}
RSF_UNSAFE=${COMPANY_UNSAFE}
RSF_PLUMB_IPNOW=6
RSF_SITE_SAFE=7

RSF_CHECK_ONLINE=100
RSF_CHECK_PARTIAL=101
RSF_CHECK_OFFLINE=102
RSF_CHECK_NORESRC=103
RSF_CHECK_ERROR=104

#
# awk script to grab the exit code from a child process run by dtrace
DTRACE_CODE_EXTRACTOR='BEGIN {code=0}/zpool_exit_code/{if($NF != "exited") {code += $NF}} END {print code}'

#
# Update path to include package directories.
PATH=${COMPANY_BIN}:${PRODUCT_BIN}:${PATH}:/sbin:/usr/sbin:/usr/bin
export PATH

export BOOL_STR_TRUE BOOL_STR_FALSE BOOL_STR_NULL
export ENFAILFAST_MICRO PROG_MHDC CACHEDIR CACHELIVEDIR

#############################################################################
# Now function definitions
#############################################################################
#
# Function:    all_pools_are_s3()
#
# Description: Checks if all pools in the current service are S3 backed
#
# Args:        None
#
# Return:      0 - Pools are backed by S3 storage
#              1 - Pools are not backed by S3
#
all_pools_are_s3()
{
    #
    # pool_idx is required by get_next_pool_vars() function.
    pool_idx=

    if is_nef; then
	while get_next_pool_vars; do
            current_pool_is_s3 || return 1
	done

	return 0
    else
	return 1
    fi
}

#############################################################################
#
# Function:    cdb_print_prop()
#
# Description: Read properties from the RSF-1 config database.
#
#
# Args:	       1) The property to read from the CDB
#              2) The default value to return if the property fails to be read
#
# Return:      Property or default
#
cdb_print_prop()
{
    if [ -x ${CDB_EXEC} -a -f ${CDB_PATH} ] ; then
        ${CDB_EXEC} ${CDB_FLAGS} -T list_props
    fi
}

#############################################################################
#
# Function:    cdb_prop_sanity_check()
#
# Description: Decide whether a property value makes sense
#
# Args:	       1) The CDB property name
#              2) The value of the property
#              3) The default value of the property
#
# Return:      0 for OK
#              1 if property value does not make sense
#              -1 for an error
#
cdb_prop_sanity_check()
{
    prop_name=$1
    prop_val=$2
    prop_default=$3

    sane=0

    # Error if any of the args are missing
    if [ $# -lt 3 ]; then
        return 1
    fi

    # Deal with special cases first
    found=1
    case ${prop_name} in
        prop_zpool_fail_mode|prop_zpool_fail_mode_${RSF_SERVICE} )
            case ${prop_val} in
            wait|continue|panic|none|default )
                ;;
            *)
                sane=1 ;;
            esac
            ;;
        prop_halt_on_failure|prop_halt_on_timeout )
            ;;
        *)
            found=0
            ;;
    esac

    if [ ${found} -eq 1 ]; then
        return ${sane}
    fi

    # Wasn't in the list of specials, test the value based on the given default

    if [ "${prop_default}" = "${BOOL_STR_TRUE}" ] || [ "${prop_default}" = "${BOOL_STR_FALSE}" ] ; then
        # Default property either true or false - value must be either true or false
        if [ "${prop_val}" != "${BOOL_STR_TRUE}" ] && [ "${prop_val}" != "${BOOL_STR_FALSE}" ] ; then
            sane=1
        fi
    elif is_numeric ${prop_default} ; then
        # Default property is a number, so value needs to be a number
        if ! is_numeric ${prop_val} ; then
            sane=1
        fi
    fi

    # Anything else can have any value

    return ${sane}
}

#############################################################################
#
# Function:    cdb_read_prop()
#
# Description: Read properties from the RSF-1 config database. If the DB does
#              not exist or there is no exec then the defaul value is returned.
#              To get a complete property list run:
#
#                rsfcdb -d gdbm -t /opt/HAC/RSF-1/etc/.cluster.db list_props
#
# Args:	       1) The property to read from the CDB
#              2) The default value to return if the property fails to be read
#
# Return:      Property or default
#
cdb_read_prop()
{
    PROPERTY_NAME="${1}"
    DEFAULT_PROPERTY_VALUE="${2}"
    RETURN_VAL="${DEFAULT_PROPERTY_VALUE}"
    UPPER_PROPERTY_NAME=`echo "${PROPERTY_NAME}"|tr '[a-z]' '[A-Z]'`

    if [ -x ${CDB_EXEC} -a -f ${CDB_PATH} ] ; then
        CDB_VALUE=`${CDB_EXEC} ${CDB_FLAGS} read ${PROPERTY_NAME}`
        #
        # Test for TRUE or FALSE or NUMBER
        if cdb_prop_sanity_check ${PROPERTY_NAME} ${CDB_VALUE} "${DEFAULT_PROPERTY_VALUE}"; then
            RETURN_VAL=`echo "${CDB_VALUE}"|sed 's/ //'g`
        fi
    fi

    #
    # Now perform an indirect assignment of the property name to the value we have selected.
    eval "${UPPER_PROPERTY_NAME}=\${RETURN_VAL}"
    eval "export ${UPPER_PROPERTY_NAME}"
}

#############################################################################
#
# Function:    cleanup()
#
# Description: Clean up a text string, leaving only alphanumerics and the - char
#
# Args:        1) The string to be cleaned
#
# Return:      Clean string
#
cleanup()
{
	_string="$*"
	echon ${_string} | tr -d -c '[:alnum:]-.'
}

#############################################################################
#
# Function:    cleanup_rsf_pid_dir()
#
# Description: Removes any PID/etc files from the RSF-1 run directory
#
# Args:        None
#
# Return:      Nothing
#
cleanup_rsf_pid_dir()
{
    if [ -d "${PRODUCT_RUN}" ]; then
        rm -f ${PRODUCT_RUN}/*
    fi
}

###################################################################################
#
# Function:    current_pool_is_s3()
#
# Description: To be used with get_next_pool_vars - checks if the current
#              pool is using S3 backend storage
#
# Args:        None
#
# Return:      0 - Pool is backed by S3 storage
#              1 - Pool is not backed by S3
#
current_pool_is_s3()
{
    [ -n "${S3_ACCOUNT}" -a -n "${S3_BUCKET}" ]
    return $?
}

#############################################################################
#
# Function:    dated_echo()
#
# Description: Output syslog-style message for log files,needs:
#              ${service} contains the service name, or Service_Undefined
#              ${script}  contains the name of the script, or Script_Undefined
#
# Args:	       1) The string to output
#
# Return:      The string with added date, service and script name
#
echo -e | grep '\-e' >/dev/null
[ $? -eq 0 ] && ECHO_E_OPTION="false" || ECHO_E_OPTION="true"
dated_echo()
{
    service="${service:-Service_Undefined}"
    script="${script:-Script_Undefined}"

    BEGIN=;END=;ARG=
    RED="\033[0;31m"
    YELLOW="\033[3;33m"
    GREEN="\033[2;32m"
    STD="\033[0m"

    if [ $# -gt 1 ]; then
        LAST_ARG="`eval echo \$\{${#}\}`"
        case ${LAST_ARG} in
            ERROR )
                BEGIN=${RED};;
            WARN )
                BEGIN=${YELLOW};;
            SUCCESS )
                BEGIN=${GREEN};;
        esac
        if [ -n "${BEGIN}" ];then
            args="${*}"
            set -- "`echo ${args} | sed 's/ [A-Z][A-Z]*$//g'`" #Remove last argument
            END=${STD}
            [ "${ECHO_E_OPTION}" = "true" ] && ARG='-e'
        fi
    fi
    date_now=`date +'%b %d %H:%M:%S'`
    echo ${ARG} "[$$ ${date_now}] [${service} ${script}] ${BEGIN}$*${END}"
}

#############################################################################
#
# Function:    echorev()
#
# Description: Output PRODUCT name and version.
#
# Args:        None
#
# Return:      Nothing
#
echorev()
{
    echo "${PRODUCT} Release ${PRODUCT_VERSION}"
}


###################################################################################
#
# Function:    getpid()
#
# Description: Get process ID of a given process
#
# Args:        1) Process name
#
# Return:      1(shell false) if no process
#              0(shell true)  if PID found
getpid()
{
    if [ $# -ne 1 ]; then
        echo "getpid called with incorrect arguments!" >&2
        return 2
    fi

    if is_freebsd; then
        # Different args for FreeBSD
        PS_OPTS="-x -o pid,command"
    else
        # These work on Solaris and Linux
        PS_OPTS="-e -o pid,args"
    fi

    PROG=`basename $1`

    PID="`ps ${PS_OPTS} | grep ${PROG} | grep -v grep | while read pid proc arg1 arg2 other;
          do
              if [ ${PROG} = \"\`basename ${proc}\`\" ]; then
                  printf \"%s \" \"${pid}\"
              elif [ -n ${arg1} ] && [ ${PROG} = \"\`basename ${arg1}\`\" ]; then
                  printf \"%s \" \"${pid}\"
              fi
          done`"

    if [ -z "${PID}" ]; then
        return 1
    else
        echo ${PID}
        return 0
    fi
}

###################################################################################
#
# Function:    get_next_pool_vars()
#
# Description: Iterator for looping through all pools in a service
#
# Args:        None
#
# Return:      0 - Variables have been set for the next pool
#              1 - No variables set (i.e. we have reached the end of the pool list)
#
get_next_pool_vars()
{
    #
    # Unset all variables
    #
    POOL_NAME=
    POOL_GUID=
    S3_ACCOUNT=
    S3_BUCKET=
    #
    # Increment the pool index - if it isn't set already, set it to 0
    #
    if [ -z "${pool_idx}" ]; then
        pool_idx=0
    else
        if ! pool_idx=`expr ${pool_idx} + 1`; then
            #
            # expr failed - pool_idx is set but not a number
            #
            pool_idx=0
        fi
    fi
    #
    # Special case - when service name is Cn_NONE, skip index 0
    #
    if [ ${pool_idx} -eq 0 ]; then
        case "${RSF_SERVICE}" in
            C[0-9]_NONE | C[0-9][0-9]_NONE | nest_syspool)
                pool_idx=1
                ;;
        esac
    fi
    #
    # Now pull out the config file environment variables
    #
    case ${pool_idx} in
        0)
            #
            # Pool name is service name
            #
            POOL_NAME=${RSF_SERVICE}
            #
            # Pool GUID is set by separate variable (or unset)
            #
            if [ -n "${ZPOOL_GUID}" ]; then
                POOL_GUID="${ZPOOL_GUID}"
            elif [ -n "${SERVICE_ZPOOL_GUID}" ]; then
                POOL_GUID="${SERVICE_ZPOOL_GUID}"
            fi
            #
            # S3 account and bucket
            #
            S3_ACCOUNT=`echo ${S3_ACCESS} | cut -d '|' -f 1`
            S3_BUCKET=`echo ${S3_ACCESS} | cut -d '|' -f 2`
            ;;
        *)
            #
            # This is an additional pool
            # Should have variable ADDITIONAL_ZPOOL_n = <pool> <guid>
            #
            eval add_var=\$ADDITIONAL_ZPOOL_${pool_idx}
            POOL_NAME=`echo ${add_var} | cut -d ' ' -f 1`
            POOL_GUID=`echo ${add_var} | cut -d ' ' -f 2`
            add_var=
            #
            # We could have an old syntax for additional pools
            #
            eval guid_var=\$ADDITIONAL_ZPOOL_${pool_idx}_GUID
            if [ -n "${guid_var}" ]; then
                POOL_GUID=${guid_var}
            fi
            #
            # S3 account and bucket
            #
            poolname_corrected=`echo ${POOL_NAME} | tr '-' '_'`
            eval s3_var=\$ADDITIONAL_S3_ACCESS_${poolname_corrected}
            S3_ACCOUNT=`echo ${s3_var} | cut -d '|' -f 1`
            S3_BUCKET=`echo ${s3_var} | cut -d '|' -f 2`
            s3_var=
            ;;
    esac
    #
    # Return true (0) only if we have set the pool name variable
    #
    if [ -n "${POOL_NAME}" ]; then
        return 0
    else
        return 1
    fi
}

#############################################################################
#
# Function:    is_freebsd()
#
# Description: Check if we are running on a freebsd appliance
#
# Args:        None
#
# Return:      0 if we are - 1 if we are not
#
is_freebsd()
{
    if uname | grep -i 'FreeBSD' >/dev/null; then
        return 0
    else
        return 1
    fi
}

#############################################################################
#
# Function:    is_metroha()
#
# Description: Check if this is a MetroHA (NEF) cluster
#              Caches the result for future calls because the
#              check binary could be slow.
#
# Args:        None
#
# Return:      0 if we are - 1 if we are not
#
is_metroha()
{
    if is_nef; then
        #
        # If rsfexec has already set IS_METROHA, use that value to save time
        #
        case ${IS_METROHA} in
            ${BOOL_STR_TRUE})
                return 0 ;;
            ${BOOL_STR_FALSE})
                return 1 ;;
        esac
        #
        # Call the Python function is_metroha()
        ${HAC_PYTHON} ${PROG_RSFPYUTILS} --isMetroHA 2>/dev/null
    else
	return 1
    fi
}

#############################################################################
#
# Function:    is_nef()
#
# Description: Check if we are running on a nexentastor 3 appliance
#
# Args:        None
#
# Return:      0 if we are - 1 if we are not
#
is_nef()
{
    if [ -d /usr/nef/bin ] || grep 'NexentaStor 5' /etc/release >/dev/null 2>&1; then
        return 0
    else
        return 1
    fi
}

#############################################################################
#
# Function:    is_numeric()
#
# Description: Test if first arg is numeric or not
#
# Args:	       1) The value to test
#
# Return:      0 for numeric
#              1 for non numeric
#
is_numeric()
{
    case "${1}" in
        ''|*[!-0-9]*|[-0-9]*-* )
	    return 1;;  # is null or contains non-numeric or '-' not in front
    esac
    return 0
}

#############################################################################
#
# Function:    is_nexenta_3()
#
# Description: Check if we are running on a nexentastor 3 appliance
#
# Args:        None
#
# Return:      0 if we are - 1 if we are not
#
is_nexenta_3()
{
    if grep 'Open Storage Appliance v*3' /etc/release >/dev/null 2>&1; then
        return 0
    else
        return 1
    fi
}

#############################################################################
#
# Function:    is_nexenta_4()
#
# Description: Check if we are running on a nexentastor 4 appliance
#
# Args:        None
#
# Return:      0 if we are - 1 if we are not
#
is_nexenta_4()
{
    if grep 'Open Storage Appliance v*4' /etc/release >/dev/null 2>&1; then
        return 0
    else
        return 1
    fi
}

#############################################################################
#
# Function:    is_nexenta()
#
# Description: Check if we are running on a nexentastor appliance
#
# Args:        None
#
# Return:      0 if we are - 1 if we are not
#
is_nexenta()
{
    if is_nef || is_nexenta_4 || is_nexenta_3 || [ -d /var/lib/nza ]; then
	return 0
    else
	return 1
    fi
}

#############################################################################
#
# Function:    is_vmware()
#
# Description: Check if we have a vcenter configuration
#
# Args:        None
#
# Return:      0 if we are - 1 if we are not
#
is_vmware()
{
    if [ -f ${VCENTRE_CONFIG} ] ; then
        return 0
    else
        return 1
    fi
}

#############################################################################
#
# Function:    match()
#
# Description: A filename globbing style pattern match with no external
#	       programme calls. Beware that $2 must not be quoted ("$2") or
#	       the construct fails. Thus $2 may not contain special
#	       characters, eg. spaces. Alternate patterns "{p1,p2}" are also
#	       unsupported.
#
# Args:        1) The string to search for the pattern in
#              2) The pattern to search for
#
# Return:      Is true (0) for a match, false (1) if not
#
match()
{
    case "$1" in
      $2) return 0 ;;
    esac
    return 1
}

#############################################################################
#
# Function:    notify_watchers()
#
# Description: Notify any interested parties about something happening in the
#              cluster. Note that the argument machine=`hostname` is always
#              added to the NMS notify call.
#              When the call is a critical error, then the last 10 lines of the
#              RSF-1 log are appended and passed in the variable logtail.
#
# Args:        1)  A string identifying the level of the message (see warning
#		   strings below)
#              2)  A string identifying the action, for example "RSF_ZFS_IMPORT"
#              3*) The remaning arguments to the watcher
#
# Return:      Nothing
#
# Known identifiers. The names in [] after the description are the arguments
# the notifier will be called with in the form 'script=<script_name>'.
# So [ipface,vip] means (for example) the call will be:
#
#  event-notify RSF_NETIF_UP machine=`hostname` ipface=bge0 vip=192.168.89.3
#
# Name			  Called when...			  Arguments
# RSF_ZFS_IMPORT          a ZFS pool is being imported            [service]
# RSF_ZFS_IMPORT_FAILED   a ZFS pool being imported fails         [service]
# RSF_ZFS_EXPORT          a ZFS pool is being exported            [service]
# RSF_ZFS_EXPORT_FAILED   a ZFS pool being exported fails         [service]
# RSF_NETIF_UP            the VIP is brought up                   [ipface,vip]
# RSF_NETIF_UP_FAIL       bringing up the interface fails         [ipface,vip]
# RSF_NETIF_DOWN          the VIP is brought down                 [ipface,vip]
# RSF_EXEC_SCRIPT         running any of the start/stop scripts   [script,state,attempt]
# RSF_AUTOSVC_RESTORED    restoring auto-* services from backup   [service]
#
# Warning strings for event notifier.
LOG_CRIT="LOG_CRIT"	# Critical conditions, such as hard device errors or fs export failure
LOG_WARN="LOG_WARN"	# Warning messages.
LOG_INFO="LOG_INFO"	# Informational messages.
ZFS_INFO="ZFS_INFO"     # ZFS actions information message
ZFS_CRIT="ZFS_CRIT"     # ZFS critical condition (export/import failure)

notify_watchers()
{

    if [ "${PROP_EVENT_NOTIFY}" = "${BOOL_STR_TRUE}" ] ; then
        #
        # Notifier itself RSF_EVENT_NOTIFY.
        NMS_NOTIFY=${RSF_EVENT_NOTIFY:-"/opt/HAC/RSF-1/bin/event_notifier"}

        LAST=""

        if [ $# -lt 2 ] ; then
	    return
        fi

        LEVEL=$1
        shift

        ACTION=$1
        shift

        if [ x"${LEVEL}" = x"${LOG_CRIT}" ] ; then
	    if [ -f "${PRODUCT_LOGDIR}/${RSF1_LOG}" ] ; then
		#
		# NEX-10537 - last 10 lines never make it all the way to the log, so
		# just sending last line and removing newlines (as this was the request
		# from NEX-10537.
	        LAST=`tail -1 ${PRODUCT_LOGDIR}/${RSF1_LOG}|sed -e 's/\n//g' -e 's/'$(printf "\x1b")'\[[0-9];*[0-9]*[mK]//g'`
	    fi
        fi

        if [ -x "${NMS_NOTIFY}" ] ; then
	    if [ ! -z "${LAST}" ] ; then
	        ${NMS_NOTIFY} "${LEVEL}" "${ACTION}" "machine=`hostname`" "logtail='${LAST}'" $* & >/dev/null 2>&1
	    else
	        ${NMS_NOTIFY} "${LEVEL}" "${ACTION}" "machine=`hostname`" $* & >/dev/null 2>&1
	    fi
        fi
    fi
}

###################################################################################
#
# Function:    rc_exit()
#
# Description:
#              This function should be called on exit from HA service RC scripts.
#              It needs to:
#               - print any error to the log file
#               - print any error to the service "broken reason" file
#               - exit with the code given
#
# Args:
#              service name
#              code
#              message
#
# Return:      None (exits)
#
rc_exit()
{
    svc=$1
    shift
    code=$1
    shift
    msg="$*"

    if [ -n "${msg}" ]; then
        case ${code} in
            ${RSF_ABORT}|${RSF_SAFE}|${RSF_UNSAFE}|${RSF_SITE_SAFE})
                #
                # Write message to broken log if there is a message
                #
                [ -n "${msg}" ] && echo "${msg}" > ${BROKEN_LOG}/${svc}.log
                dated_echo "${msg} - Exiting (${code})" ERROR
                #
                # May need to override code depending on DB prop
                #
                service_halt ${svc}
                ;;
            ${RSF_WARN})
                dated_echo "${msg} - Exiting (${code})" WARN
                ;;
            *)
                dated_echo "${msg} - Exiting (${code})"
                ;;
        esac
    fi

    exit ${code}
}

#############################################################################
#
# Function:    rotatelogs()
#
# Description: Rotate the log files by one numeric suffix, by default log.9 is
#              overwritten. The file .keeplogs in the log file directory can be
#              used to extent the number of log files to keep.
#
# Args:        1) Directory where log files reside.
#              2) File name of log file without trailing revision number.
#
# Return:      Nothing
#
rotatelogs()
{
    LogfileDirectory=$1
    LogfileName=$2
    CURRENT=`pwd`

    mkdir -p "${LogfileDirectory}"

    if [ ! -d "${LogfileDirectory}" ] ; then
	echo "rotatelogs: log file directory does not exist: ${LogfileDirectory}"
	return
    fi

    #
    # Number of revisions to keep; .keeplogs file can change the default from 9.
    i=9
    if [ -f "${LogfileDirectory}/.keeplogs" ] ; then
	i=`cat ${LogfileDirectory}/.keeplogs|sed 's/[^0-9]//g'`

	if [ -z "$i" ] ; then
	    i=9
	    echo "rotatelogs: .keeplogs file does not contain a number between 1-99, setting to default of 9"
	else
	    case "$i" in
		[0-9]*) ;;
		*) i=9; echo "rotatelogs: .keeplogs file does not contain a number between 1-99, setting to default of 9";;
            esac
        fi
    fi

    cd ${LogfileDirectory}

    while [ $i -ge 1 ]
      do
      j=`expr $i - 1`
      test -f ${LogfileName}.$j && mv ${LogfileName}.$j ${LogfileName}.$i >/dev/null 2>&1
      i=$j
    done

    test -f ${LogfileName} && mv ${LogfileName} ${LogfileName}.$i >/dev/null 2>&1

    cd ${CURRENT}
}

###################################################################################
#
# Function:    service_halt()
#
# Description:
#              This function should be called if a service fails in some way.
#              It will halt the service cluster-wide by calling exit with code
#              RSF_UNSAFE.
#              This only happens though if the halt property is set in the database.
#              If not, this function is a no-op.
#
# Args:
#              service name
#
# Return:      None
#
service_halt()
{
	#
	# Service name is $1 but avoid setting variable.
	# All shell variables are global and we want this
	# function to be a no-op most of the time.
	#

	#
	# Make sure we don't exit if no service was specified
	#
	[ $# -eq 0 ] && return

	#
	# $2 is optional - if it is set to "timeout" then
	# we need to look at PROP_HALT_ON_TIMEOUT instead of
	# PROP_HALT_ON_FAILURE
	#
	if [ "$2" = "timeout" ]; then
		halt_propname=PROP_HALT_ON_TIMEOUT
		halt_propval=${PROP_HALT_ON_TIMEOUT}
	else
		halt_propname=PROP_HALT_ON_FAILURE
		halt_propval=${PROP_HALT_ON_FAILURE}
	fi

	#
	# PROP_HALT_ON_FAILURE could be "true", "false" or
	# a space separated list of service names
	#
	case ,${halt_propval}, in
		,${BOOL_STR_TRUE}, | *,${1},* )
			echo "Halting service ${svc} (${halt_propname} = ${halt_propval})"
			exit ${RSF_UNSAFE}
			;;
	esac
}

#############################################################################
#
# Function:    split()
#
# Description: Split a string of $IFS seperated words into the individual
#              words, and assign them to a list of variables. Extra words or
#              variables are silently ignored.
#              eg.	"split 'hello Fred' greeting person"
#              sets greeting=hello, person=Fred
#              but	"split 'hello Fred Bloggs' greeting person
#              sets greeting=hello, person=Fred BUT ignors Bloggs
#
# Args:        1) The string to split (i.e. 'hello world')
#              *) Names of variables to set the split string to.
#
# Return:      Variables set as per description.
#
split()
{
    split_wordlist="$1"		# Grab the wordlist
    shift			# Remainder of arguments start at $1.

    for split_word in $split_wordlist; do
	match x"$1" x && break	# No more variables to set
	eval "$1=$split_word"	# Set variable to next word
	shift			# Shift back arguments for next $1
    done
}

#############################################################################
#
# Function:    systype()
#
# Description: Set the global variables COMPANY_OS COMPANY_OS_REL COMPANY_ARCH
#              to represent the current running machine, and modify type where
#              necessary (i.e. sunos i386 => x86).
#
# Args:        None
#
# Return:      Nothing
#
systype()
{
    COMPANY_OS_ORIG=`uname -s`
    osname=`uname -s | tr '[A-Z]' '[a-z]'`
    osrel=`uname -r | tr '[A-Z]' '[a-z]'`
    arch=`uname -m | tr '[A-Z]' '[a-z]' | sed -e "s/i.*86/x86/g"`

    case "${osname}" in
	hp-ux)
	    arch=PARISC
	    ;;

	aix)
	    osrel="`uname -v`.${osrel}"
	    arch=ppc
	    ;;

	freebsd)
	    osrel=`echo ${osrel} | tr -d '[a-z-]'`
	    ;;

	openbsd)
	    ;;

	linux)
	    ;;

	darwin)
	    ;;

	sunos)
	    arch=`uname -p | tr '[A-Z]' '[a-z]' | sed -e "s/i.*86/x86/g"`
	    ;;
    esac

    COMPANY_OS=${osname}
    COMPANY_OS_REL=${osrel}
    COMPANY_ARCH=${arch}

    export COMPANY_OS COMPANY_OS_REL COMPANY_ARCH COMPANY_OS_ORIG
}

###################################################################################
#
# Function:    xor()
#
# Description: Generic handler for question with two possible answers
#
# Args:        1) ans1
#              2) ans2
#              3) default
#
# Return:      1(shell false) for ans1
#              0(shell true)  for ans2
#
# Usage: answer1 ansert2 default
xor()
{
    while true; do
	if read answer; then
	    answer=`echo "r-$answer" | tr '[A-Z]' '[a-z]'`
	else
	    answer=`echo "d-$3" | tr '[A-Z]' '[a-z]'`
	fi

	case "${answer}" in
	    r-${1}*) return 1 ;;
	    r-${2}*) return 0 ;;
	    d-${1}*) echo "$3"; return 1 ;;
	    d-*)     echo "$3"; return 0 ;;
	    *)       printf "%s" "*** Please enter ${1} or ${2}: " ;;
	esac
    done
}

#############################################################################
#
# Function:    cdb_check_sanity()
#
# Description: Check if data can be read from RSFCDB
#
#
# Args:        None
#
# Return:      0 ) Success.
#              1 ) DB failed sanity check
#              2 ) No CDB path found and/or database not found.
#
cdb_check_sanity()
{
    if [ -x ${CDB_EXEC} -a -f ${CDB_PATH} ] ; then
        ${CDB_EXEC} ${CDB_FLAGS} -s list_props

        if [ $? -eq 0 ]; then
            return 0
        else
            return 1
        fi
    fi
    return 2
}

cdb_check_sanity

if [ $? -eq 0 ]; then
    if [ "${DB_PROPS_DEFINED}" != "${BOOL_STR_TRUE}" ]; then
        cdb_read_prop prop_abort_unfenced_import                "${PROP_ABORT_UNFENCED_IMPORT__VALUE}"
        cdb_read_prop prop_additional_common_directories        "${PROP_ADDITIONAL_COMMON_DIRECTORIES__VALUE}"
        cdb_read_prop prop_cluster_hook                         "${PROP_CLUSTER_HOOK__VALUE}"
        cdb_read_prop prop_comstar_standby_luns_before_failover "${PROP_COMSTAR_STANDBY_LUNS_BEFORE_FAILOVER__VALUE}"
        cdb_read_prop prop_comstar_support                      "${PROP_COMSTAR_SUPPORT__VALUE}"
        cdb_read_prop prop_event_notify                         "${PROP_EVENT_NOTIFY__VALUE}"
        cdb_read_prop prop_fping_retries                        "${PROP_FPING_RETRIES__VALUE}"
        cdb_read_prop prop_gui_default_cluster_name             "${PROP_GUI_DEFAULT_CLUSTER_NAME__VALUE}"
        cdb_read_prop prop_gui_default_inittimeout              "${PROP_GUI_DEFAULT_INITTIMEOUT__VALUE}"
        cdb_read_prop prop_gui_default_runtimeout               "${PROP_GUI_DEFAULT_RUNTIMEOUT__VALUE}"
        cdb_read_prop prop_gui_diagnostics_test                 "${PROP_GUI_DIAGNOSTICS_TEST__VALUE}"
        cdb_read_prop prop_gui_disable_online_help              "${PROP_GUI_DISABLE_ONLINE_HELP__VALUE}"
        cdb_read_prop prop_gui_enable_comstar_browser           "${PROP_GUI_ENABLE_COMSTAR_BROWSER__VALUE}"
        cdb_read_prop prop_gui_enable_diagnostics               "${PROP_GUI_ENABLE_DIAGNOSTICS__VALUE}"
        cdb_read_prop prop_gui_enable_fastfail                  "${PROP_GUI_ENABLE_FASTFAIL__VALUE}"
        cdb_read_prop prop_gui_enable_ipv6                      "${PROP_GUI_ENABLE_IPV6__VALUE}"
        cdb_read_prop prop_gui_plugin_version                   "${PROP_GUI_PLUGIN_VERSION__VALUE}"
        cdb_read_prop prop_gui_failover_state_sticky            "${PROP_GUI_FAILOVER_STATE_STICKY__VALUE}"
        cdb_read_prop prop_gui_javascript_debug                 "${PROP_GUI_JAVASCRIPT_DEBUG__VALUE}"
        cdb_read_prop prop_gui_plugin_version                   "${PROP_GUI_PLUGIN_VERSION__VALUE}"
        cdb_read_prop prop_gui_res_drive_support                "${PROP_GUI_RES_DRIVE_SUPPORT__VALUE}"
        cdb_read_prop prop_gui_serial_hb_support                "${PROP_GUI_SERIAL_HB_SUPPORT__VALUE}"
        cdb_read_prop prop_mhdc_stec_skip                       "${PROP_MHDC_STEC_SKIP__VALUE}"
        cdb_read_prop prop_mhdc_use_failfast                    "${PROP_MHDC_USE_FAILFAST__VALUE}"
        cdb_read_prop prop_mhdc_use_pgr3                        "${PROP_MHDC_USE_PGR3__VALUE}"
        cdb_read_prop prop_pgr3_drive_count                     "${PROP_PGR3_DRIVE_COUNT__VALUE}"
        cdb_read_prop prop_pool_hook                            "${PROP_POOL_HOOK__VALUE}"
        cdb_read_prop prop_python_rpc_server_portno             "${PROP_PYTHON_RPC_SERVER_PORTNO__VALUE}"
        cdb_read_prop prop_rsf_zfs_event_logging                "${PROP_RSF_ZFS_EVENT_LOGGING__VALUE}"
        cdb_read_prop prop_scsi2_drive_count                    "${PROP_SCSI2_DRIVE_COUNT__VALUE}"
        cdb_read_prop prop_scsi2_retry_count                    "${PROP_SCSI2_RETRY_COUNT__VALUE}"
        cdb_read_prop prop_res_drive_test_retries               "${PROP_RES_DRIVE_TEST_RETRIES__VALUE}"
        cdb_read_prop prop_res_drive_test_delay                 "${PROP_RES_DRIVE_TEST_DELAY__VALUE}"
        cdb_read_prop prop_scsi2_threaded_reserve               "${PROP_SCSI2_THREADED_RESERVE__VALUE}"
        cdb_read_prop prop_scsi2_threaded_reserve_count         "${PROP_SCSI2_THREADED_RESERVE_COUNT__VALUE}"
        cdb_read_prop prop_skip_pgr3_log_and_cache              "${PROP_SKIP_PGR3_LOG_AND_CACHE__VALUE}"
        cdb_read_prop prop_skip_vip_check                       "${PROP_SKIP_VIP_CHECK__VALUE}"
        cdb_read_prop prop_ssh_bound                            "${PROP_SSH_BOUND__VALUE}"
        cdb_read_prop prop_sync_rsfcdb                          "${PROP_SYNC_RSFCDB__VALUE}"
        cdb_read_prop prop_use_zfs_cache                        "${PROP_USE_ZFS_CACHE__VALUE}"
        cdb_read_prop prop_zpool_dtrace_import_export           "${PROP_ZPOOL_DTRACE_IMPORT_EXPORT__VALUE}"
        cdb_read_prop prop_zpool_dtrace_needed_timeout_seconds  "${PROP_ZPOOL_DTRACE_NEEDED_TIMEOUT_SECONDS__VALUE}"
        cdb_read_prop prop_zpool_export_option_c                "${PROP_ZPOOL_EXPORT_OPTION_C__VALUE}"
        cdb_read_prop prop_zpool_fail_mode                      "${PROP_ZPOOL_FAIL_MODE__VALUE}"
        cdb_read_prop prop_zpool_import_option_m                "${PROP_ZPOOL_IMPORT_OPTION_M__VALUE}"
        cdb_read_prop prop_zpool_threads_number                 "${PROP_ZPOOL_THREADS_NUMBER__VALUE}"
        cdb_read_prop prop_zpool_threads_option_t               "${PROP_ZPOOL_THREADS_OPTION_T__VALUE}"
        cdb_read_prop prop_zpool_sync_cache                     "${PROP_ZPOOL_SYNC_CACHE__VALUE}"
        cdb_read_prop prop_res_drive_wait                       "${PROP_RES_DRIVE_WAIT__VALUE}"
        cdb_read_prop prop_nfs_failover_support                 "${PROP_NFS_FAILOVER_SUPPORT__VALUE}"
        cdb_read_prop prop_disable_site_safe                    "${PROP_DISABLE_SITE_SAFE__VALUE}"
        cdb_read_prop prop_force_ifconfig                       "${PROP_FORCE_IFCONFIG__VALUE}"
        cdb_read_prop prop_disable_drive_monitor                "${PROP_DISABLE_DRIVE_MONITOR__VALUE}"
        cdb_read_prop prop_auto_update_mon_exprs                "${PROP_AUTO_UPDATE_MON_EXPRS__VALUE}"

        # If we want ot use DTrace for zpool imports and exports, make sure it works properly!
        #if [ "${PROP_ZPOOL_DTRACE_IMPORT_EXPORT}" = "${BOOL_STR_TRUE}" ] && ! check_dtrace; then
        #	PROP_ZPOOL_DTRACE_IMPORT_EXPORT=${BOOL_STR_FALSE}
        #fi

        DB_PROPS_DEFINED="${BOOL_STR_TRUE}"
        export DB_PROPS_DEFINED
    fi
fi

#############################################################################
# Inline code goes here
#############################################################################
#
# See if the contract runner is available
if [ -x "/usr/bin/ctrun" ] ; then
    CTRUN="/usr/bin/ctrun -l none "
else
    CTRUN=""
fi
export CTRUN

#
# STMFHA path
if [ -x "/opt/HAC/RSF-1/bin/stmfha" ] ; then
    PROG_STMFHA="/opt/HAC/RSF-1/bin/stmfha"
else
    if [ -x "/usr/bin/stmfha" ] ; then
        PROG_STMFHA="/usr/bin/stmfha"
    else
        PROG_STMFHA=""
    fi
fi
export PROG_STMFHA

#
# Test and read product version.
if [ -f "${PRODUCT_DIR}/.version" ]; then
    PRODUCT_VERSION="`cat ${PRODUCT_DIR}/.version`"
fi

#
# Check if dtrace works and, if not, block it's use.
# The property PROP_DTRACE_IS_BROKEN is set on every install
# as part of <not sure>
if [ ! -z "${PROP_DTRACE_IS_BROKEN}" -a "${PROP_DTRACE_IS_BROKEN}" = "${BOOL_STR_TRUE}" ] ; then
    PROP_ZPOOL_DTRACE_IMPORT_EXPORT=${BOOL_STR_FALSE}
fi

systype

PREFIX=${BASEDIR:-/}
INCLUDED_PACKAGE_SH=""

case ${PREFIX} in
    */)
	. ${PREFIX}opt/HAC/bin/package.sh
	;;

    *)
	. ${PREFIX}/opt/HAC/bin/package.sh
	;;
esac
