#!/bin/bash
DATASET="10054"
INC=1

POOL=$1
if [ $# -ne 1 ]; then
    echo "`basename $0` called with incorrect number of arguments - should be 1" >&2
elif ! [ -d /volumes/${POOL} ]; then
    echo "Unable to find expected mount point /volumes/${POOL}" >&2
fi

dated_echo()
{
    SLEEP=0
    MSG=""

    if [ "${1}" == '-s' ] ; then
        shift
        SLEEP=30
        MSG="- sleeping for 30 seconds..."
    fi

    date_now=`date +'%b %d %H:%M:%S'`
    echo "[$0 $$ ${date_now}] $* ${MSG}"
    
    if [ ${SLEEP} -ne 0 ] ; then
        sleep ${SLEEP}
    fi
}

do_zfs_fastflush()
{
    dated_echo "Enabling zfs_fastflush"
    echo zfs_fastflush/W1 | mdb -kw
}

do_filesystem_destroy()
{
    dated_echo "Destroying previous filesystems"

    for i in {1..80}
    do
        if [ -d /volumes/${POOL}/${DATASET}-$i ] ; then
            zfs destroy ${POOL}/${DATASET}-$i &
        fi
    done
    wait

    dated_echo -s "Done destroying filesystems"
}

do_filesystem_create()
{
    dated_echo "Setting up new filesystems"

    for i in {1..80}
    do
        /usr/bin/echo "${i} \c"
        zfs create ${POOL}/${DATASET}-$i
        zfs set compression=off ${POOL}/${DATASET}-$i
        zfs set recordsize=4K ${POOL}/${DATASET}-$i
    done

    /usr/bin/echo ''
    dated_echo -s "File system setup done"
}

do_copy_dd_threads()
{
    dated_echo "Starting copy threads"
    for i in {1..80}
    do
        dd if=/dev/zero of=/volumes/${POOL}/${DATASET}-$i/file bs=128k count=10240 skip=`expr \( $i - 1 \) \* 5120` &
    done
    wait
    dated_echo -s "Done copying from pool"
}

do_read_dd_threads()
{
    echo Starting read threads
    for i in {1..80}
    do
        dd if=/volumes/${POOL}/${DATASET}-$i/file bs=128k count=10240 of=/dev/null &
    done
    wait
    dated_echo -s "Done reading from pool"
}
# ./txg_sync_thread.d > txg_sync_thread.out &

do_timed_export()
{
    dated_echo "Starting timed export"
    time zpool export ${POOL}
    dated_echo -s "Done export"
}

do_timed_import()
{
    dated_echo "Starting timed import"
    time zpool import ${POOL}
    dated_echo "Done import."
}

do_test()
{
    do_filesystem_destroy
    do_filesystem_create
    do_copy_dd_threads
    do_read_dd_threads
    do_timed_export
    do_timed_import
}

do_zfs_fastflush

do_test

exit 0

while : ; do
    dated_echo "Running test ${INC}"
    do_test
    INC=`expr ${INC} + 1`
done
